@@ -1,6 +1,3 @@
-## Automatically adapted for scipy Oct 21, 2005 by
-
-# Author: Travis Oliphant
 
 __all__ = ['fixed_quad','quadrature','romberg','trapz','simps','romb',
            'cumtrapz','newton_cotes','composite']
@@ -9,38 +6,43 @@ from scipy.special.orthogonal import p_roots
 from scipy.special import gammaln
 from numpy import sum, ones, add, diff, isinf, isscalar, \
      asarray, real, trapz, arange, empty
-import scipy as sp
 import numpy as np
+import math
 
 def fixed_quad(func,a,b,args=(),n=5):
-    """Compute a definite integral using fixed-order Gaussian quadrature.
+    """
+    Compute a definite integral using fixed-order Gaussian quadrature.
 
-  Description:
+    Integrate `func` from a to b using Gaussian quadrature of order n.
 
-    Integrate func from a to b using Gaussian quadrature of order n.
+    Parameters
+    ----------
+    func : callable
+        A Python function or method to integrate (must accept vector inputs).
+    a : float
+        Lower limit of integration.
+    b : float
+        Upper limit of integration.
+    args : tuple, optional
+        Extra arguments to pass to function, if any.
+    n : int, optional
+        Order of quadrature integration. Default is 5.
 
-  Inputs:
+    Returns
+    -------
+    val : float
+        Gaussian quadrature approximation to the integral
 
-    func -- a Python function or method to integrate
-            (must accept vector inputs)
-    a -- lower limit of integration
-    b -- upper limit of integration
-    args -- extra arguments to pass to function.
-    n -- order of quadrature integration.
-
-  Outputs: (val, None)
-
-    val -- Gaussian quadrature approximation to the integral.
-
-  See also:
-    
-    quad - adaptive quadrature using QUADPACK
-    dblquad, tplquad - double and triple integrals
-    romberg - adaptive Romberg quadrature
-    quadrature - adaptive Gaussian quadrature
-    romb, simps, trapz - integrators for sampled data
-    cumtrapz - cumulative integration for sampled data
+    See Also
+    --------
+    quad : adaptive quadrature using QUADPACK
+    dblquad, tplquad : double and triple integrals
+    romberg : adaptive Romberg quadrature
+    quadrature : adaptive Gaussian quadrature
+    romb, simps, trapz : integrators for sampled data
+    cumtrapz : cumulative integration for sampled data
     ode, odeint - ODE integrators
+
     """
     [x,w] = p_roots(n)
     x = real(x)
@@ -52,6 +54,30 @@ def fixed_quad(func,a,b,args=(),n=5):
     return (b-a)/2.0*sum(w*func(y,*args),0), None
 
 def vectorize1(func, args=(), vec_func=False):
+    """Vectorize the call to a function.
+
+    This is an internal utility function used by `romberg` and
+    `quadrature` to create a vectorized version of a function.
+
+    If `vec_func` is True, the function `func` is assumed to take vector
+    arguments.
+
+    Parameters
+    ----------
+    func : callable
+        User defined function.
+    args : tuple
+        Extra arguments for the function.
+    vec_func : bool
+        True if the function func takes vector arguments.
+
+    Returns
+    -------
+    vfunc : callable
+        A function that will take a vector argument and return the
+        result.
+
+    """
     if vec_func:
         def vfunc(x):
             return func(x, *args)
@@ -63,7 +89,10 @@ def vectorize1(func, args=(), vec_func=False):
             # call with first point to get output type
             y0 = func(x[0], *args)
             n = len(x)
-            output = empty((n,), dtype=y0.dtype)
+            if hasattr(y0, 'dtype'):
+                output = empty((n,), dtype=y0.dtype)
+            else:
+                output = empty((n,), dtype=type(y0))
             output[0] = y0
             for i in xrange(1, n):
                 output[i] = func(x[i], *args)
@@ -71,39 +100,52 @@ def vectorize1(func, args=(), vec_func=False):
     return vfunc
 
 def quadrature(func,a,b,args=(),tol=1.49e-8,maxiter=50, vec_func=True):
-    """Compute a definite integral using fixed-tolerance Gaussian quadrature.
-
-  Description:
+    """
+    Compute a definite integral using fixed-tolerance Gaussian quadrature.
 
     Integrate func from a to b using Gaussian quadrature
-    with absolute tolerance tol.
+    with absolute tolerance `tol`.
 
-  Inputs:
+    Parameters
+    ----------
+    func : function
+        A Python function or method to integrate.
+    a : float
+        Lower limit of integration.
+    b : float
+        Upper limit of integration.
+    args : tuple, optional
+        Extra arguments to pass to function.
+    tol : float, optional
+        Iteration stops when error between last two iterates is less than
+        tolerance.
+    maxiter : int, optional
+        Maximum number of iterations.
+    vec_func : bool, optional
+        True or False if func handles arrays as arguments (is
+        a "vector" function). Default is True.
 
-    func -- a Python function or method to integrate.
-    a -- lower limit of integration.
-    b -- upper limit of integration.
-    args -- extra arguments to pass to function.
-    tol -- iteration stops when error between last two iterates is less than
-           tolerance.
-    maxiter -- maximum number of iterations.
-    vec_func -- True or False if func handles arrays as arguments (is
-                a "vector" function ). Default is True.
+    Returns
+    -------
+    val : float
+        Gaussian quadrature approximation (within tolerance) to integral.
+    err : float
+        Difference between last two estimates of the integral.
 
-  Outputs: (val, err)
+    See also
+    --------
+    romberg: adaptive Romberg quadrature
+    fixed_quad: fixed-order Gaussian quadrature
+    quad: adaptive quadrature using QUADPACK
+    dblquad: double integrals
+    tplquad: triple integrals
+    romb: integrator for sampled data
+    simps: integrator for sampled data
+    trapz: integrator for sampled data
+    cumtrapz: cumulative integration for sampled data
+    ode: ODE integrator
+    odeint: ODE integrator
 
-    val -- Gaussian quadrature approximation (within tolerance) to integral.
-    err -- Difference between last two estimates of the integral.
-
-  See also:
-    
-    romberg - adaptive Romberg quadrature
-    fixed_quad - fixed-order Gaussian quadrature
-    quad - adaptive quadrature using QUADPACK
-    dblquad, tplquad - double and triple integrals
-    romb, simps, trapz - integrators for sampled data
-    cumtrapz - cumulative integration for sampled data
-    ode, odeint - ODE integrators
     """
     err = 100.0
     val = err
@@ -124,20 +166,41 @@ def tupleset(t, i, value):
     return tuple(l)
 
 def cumtrapz(y, x=None, dx=1.0, axis=-1):
-    """Cumulatively integrate y(x) using samples along the given axis
+    """
+    Cumulatively integrate y(x) using samples along the given axis
     and the composite trapezoidal rule.  If x is None, spacing given by dx
     is assumed.
 
-    See also:
-    
-      quad - adaptive quadrature using QUADPACK
-      romberg - adaptive Romberg quadrature
-      quadrature - adaptive Gaussian quadrature
-      fixed_quad - fixed-order Gaussian quadrature
-      dblquad, tplquad - double and triple integrals
-      romb, trapz - integrators for sampled data
-      cumtrapz - cumulative integration for sampled data
-      ode, odeint - ODE integrators
+    Parameters
+    ----------
+    y : array
+
+    x : array, optional
+
+    dx : int, optional
+
+    axis : int, optional
+        Specifies the axis to cumulate:
+
+          - -1 --> X axis
+          - 0  --> Z axis
+          - 1  --> Y axis
+
+    See Also
+    --------
+
+      quad: adaptive quadrature using QUADPACK
+      romberg: adaptive Romberg quadrature
+      quadrature: adaptive Gaussian quadrature
+      fixed_quad: fixed-order Gaussian quadrature
+      dblquad: double integrals
+      tplquad: triple integrals
+      romb: integrators for sampled data
+      trapz: integrators for sampled data
+      cumtrapz: cumulative integration for sampled data
+      ode: ODE integrators
+      odeint: ODE integrators
+
     """
     y = asarray(y)
     if x is None:
@@ -180,39 +243,57 @@ def _basic_simps(y,start,stop,x,dx,axis):
 
 
 def simps(y, x=None, dx=1, axis=-1, even='avg'):
-    """Integrate y(x) using samples along the given axis and the composite
+    """
+    Integrate y(x) using samples along the given axis and the composite
     Simpson's rule.  If x is None, spacing of dx is assumed.
 
     If there are an even number of samples, N, then there are an odd
     number of intervals (N-1), but Simpson's rule requires an even number
-    of intervals.  The parameter 'even' controls how this is handled as
-    follows:
+    of intervals.  The parameter 'even' controls how this is handled.
 
-    even='avg': Average two results: 1) use the first N-2 intervals with
-                a trapezoidal rule on the last interval and 2) use the last
-                N-2 intervals with a trapezoidal rule on the first interval
+    Parameters
+    ----------
+    y : array_like
+        Array to be integrated.
+    x : array_like, optional
+        If given, the points at which `y` is sampled.
+    dx : int, optional
+        Spacing of integration points along axis of `y`. Only used when
+        `x` is None. Default is 1.
+    axis : int, optional
+        Axis along which to integrate. Default is the last axis.
+    even : {'avg', 'first', 'str'}, optional
+        'avg' : Average two results:1) use the first N-2 intervals with
+                  a trapezoidal rule on the last interval and 2) use the last
+                  N-2 intervals with a trapezoidal rule on the first interval.
 
-    even='first': Use Simpson's rule for the first N-2 intervals with
-                  a trapezoidal rule on the last interval.
+        'first' : Use Simpson's rule for the first N-2 intervals with
+                a trapezoidal rule on the last interval.
 
-    even='last': Use Simpson's rule for the last N-2 intervals with a
-                 trapezoidal rule on the first interval.
+        'last' : Use Simpson's rule for the last N-2 intervals with a
+               trapezoidal rule on the first interval.
 
+    See Also
+    --------
+    quad: adaptive quadrature using QUADPACK
+    romberg: adaptive Romberg quadrature
+    quadrature: adaptive Gaussian quadrature
+    fixed_quad: fixed-order Gaussian quadrature
+    dblquad: double integrals
+    tplquad: triple integrals
+    romb: integrators for sampled data
+    trapz: integrators for sampled data
+    cumtrapz: cumulative integration for sampled data
+    ode: ODE integrators
+    odeint: ODE integrators
+
+    Notes
+    -----
     For an odd number of samples that are equally spaced the result is
-        exact if the function is a polynomial of order 3 or less.  If
-        the samples are not equally spaced, then the result is exact only
-        if the function is a polynomial of order 2 or less.
+    exact if the function is a polynomial of order 3 or less.  If
+    the samples are not equally spaced, then the result is exact only
+    if the function is a polynomial of order 2 or less.
 
-    See also:
-    
-      quad - adaptive quadrature using QUADPACK
-      romberg - adaptive Romberg quadrature
-      quadrature - adaptive Gaussian quadrature
-      fixed_quad - fixed-order Gaussian quadrature
-      dblquad, tplquad - double and triple integrals
-      romb, trapz - integrators for sampled data
-      cumtrapz - cumulative integration for sampled data
-      ode, odeint - ODE integrators
     """
     y = asarray(y)
     nd = len(y.shape)
@@ -269,20 +350,30 @@ def simps(y, x=None, dx=1, axis=-1, even='avg'):
     return result
 
 def romb(y, dx=1.0, axis=-1, show=False):
-    """Romberg integration using samples of a function
+    """
+    Romberg integration using samples of a function
 
-    Inputs:
+    Parameters
+    -----------
+      y : array like
+           a vector of 2**k + 1 equally-spaced samples of a function
 
-       y    -  a vector of 2**k + 1 equally-spaced samples of a fucntion
-       dx   -  the sample spacing.
-       axis -  the axis along which to integrate
-       show -  When y is a single 1-d array, then if this argument is True
-               print the table showing Richardson extrapolation from the
-               samples.
+      dx : array like
+           the sample spacing.
 
-    Output: ret
+      axis : array like?
+           the axis along which to integrate
 
-       ret  - The integrated result for each axis. 
+      show : Boolean
+           When y is a single 1-d array, then if this argument is True
+           print the table showing Richardson extrapolation from the
+           samples.
+
+    Returns
+    -----------
+
+       ret : array_like?
+          The integrated result for each axis.
 
     See also:
 
@@ -294,6 +385,7 @@ def romb(y, dx=1.0, axis=-1, show=False):
       simps, trapz - integrators for sampled data
       cumtrapz - cumulative integration for sampled data
       ode, odeint - ODE integrators
+
     """
     y = asarray(y)
     nd = len(y.shape)
@@ -405,7 +497,7 @@ def _printresmat(function, interval, resmat):
     print ''
     print '%6s %9s %9s' % ('Steps', 'StepSize', 'Results')
     for i in range(len(resmat)):
-        print '%6d %9f' % (2**i, (interval[1]-interval[0])/(i+1.0)),
+        print '%6d %9f' % (2**i, (interval[1]-interval[0])/(2.**i)),
         for j in range(i+1):
             print '%9f' % (resmat[i][j]),
         print ''
@@ -415,25 +507,83 @@ def _printresmat(function, interval, resmat):
 
 def romberg(function, a, b, args=(), tol=1.48E-8, show=False,
             divmax=10, vec_func=False):
-    """Romberg integration of a callable function or method.
+    """
+    Romberg integration of a callable function or method.
 
-    Returns the integral of |function| (a function of one variable)
-    over |interval| (a sequence of length two containing the lower and
-    upper limit of the integration interval), calculated using
-    Romberg integration up to the specified |accuracy|. If |show| is 1,
-    the triangular array of the intermediate results will be printed.
-    If |vec_func| is True (default is False), then |function| is
+    Returns the integral of `function` (a function of one variable)
+    over the interval (`a`, `b`).
+
+    If `show` is 1, the triangular array of the intermediate results
+    will be printed.  If `vec_func` is True (default is False), then `function` is
     assumed to support vector arguments.
 
-    See also:
-    
-      quad - adaptive quadrature using QUADPACK
-      quadrature - adaptive Gaussian quadrature
-      fixed_quad - fixed-order Gaussian quadrature
-      dblquad, tplquad - double and triple integrals
-      romb, simps, trapz - integrators for sampled data
-      cumtrapz - cumulative integration for sampled data
-      ode, odeint - ODE integrators
+    Parameters
+    ----------
+    function : callable
+        Function to be integrated.
+    a : float
+        Lower limit of integration.
+    b : float
+        Upper limit of integration.
+
+    Returns
+    --------
+    results  : float
+        Result of the integration.
+
+    Other Parameters
+    ----------------
+    args : tuple, optional
+        Extra arguments to pass to function. Each element of `args` will
+        be passed as a single argument to `func`. Default is to pass no
+        extra arguments.
+    tol : float, optional
+        The desired tolerance. Default is 1.48e-8.
+    show : bool, optional
+        Whether to print the results. Default is False.
+    divmax : int, optional
+        ?? Default is 10.
+    vec_func : bool, optional
+        Whether `func` handles arrays as arguments (i.e whether it is a
+        "vector" function). Default is False.
+
+    See Also
+    --------
+    fixed_quad : Fixed-order Gaussian quadrature.
+    quad : Adaptive quadrature using QUADPACK.
+    dblquad, tplquad : Double and triple integrals.
+    romb, simps, trapz : Integrators for sampled data.
+    cumtrapz : Cumulative integration for sampled data.
+    ode, odeint : ODE integrators.
+
+    References
+    ----------
+    .. [1] 'Romberg's method' http://en.wikipedia.org/wiki/Romberg%27s_method
+
+    Examples
+    --------
+    Integrate a gaussian from 0,1 and compare to the error function.
+
+    >>> from scipy.special import erf
+    >>> gaussian = lambda x: 1/np.sqrt(np.pi) * np.exp(-x**2)
+    >>> result = romberg(gaussian, 0, 1, show=True)
+    Romberg integration of <function vfunc at 0x101eceaa0> from [0, 1]
+
+    ::
+
+       Steps  StepSize  Results
+           1  1.000000  0.385872
+           2  0.500000  0.412631  0.421551
+           4  0.250000  0.419184  0.421368  0.421356
+           8  0.125000  0.420810  0.421352  0.421350  0.421350
+          16  0.062500  0.421215  0.421350  0.421350  0.421350  0.421350
+          32  0.031250  0.421317  0.421350  0.421350  0.421350  0.421350  0.421350
+
+    The final result is 0.421350396475 after 33 function evaluations.
+
+    >>> print 2*result,erf(1)
+    0.84270079295 0.84270079295
+
     """
     if isinf(a) or isinf(b):
         raise ValueError("Romberg integration only available for finite limits.")
@@ -518,38 +668,49 @@ _builtincoeffs = {
                         -16802270373,12630121616,-6625093363,3501442784,
                         -770720657,710986864,90241897], -3740727473,
         1275983280000)
-    }           
+    }
 
 def newton_cotes(rn,equal=0):
-    r"""Return weights and error coefficient for Netwon-Cotes integration.
+    """
+    Return weights and error coefficient for Newton-Cotes integration.
 
-     Suppose we have (N+1) samples of f at the positions
-       x_0, x_1, ..., x_N.  Then an N-point Newton-Cotes formula for the
-       integral between x_0 and x_N is:
+    Suppose we have (N+1) samples of f at the positions
+    x_0, x_1, ..., x_N.  Then an N-point Newton-Cotes formula for the
+    integral between x_0 and x_N is:
 
-     $\int_{x_0}^{x_N} f(x)dx = \Delta x \sum_{i=0}^{N} a_i f(x_i)
-                                + B_N (\Delta x)^{N+2} f^{N+1} (\xi)$
+    :math:`\\int_{x_0}^{x_N} f(x)dx = \\Delta x \\sum_{i=0}^{N} a_i f(x_i)
+    + B_N (\\Delta x)^{N+2} f^{N+1} (\\xi)`
 
-       where $\xi \in [x_0,x_N]$ and $\Delta x = \frac{x_N-x_0}{N}$ is the
-       averages samples spacing. 
+    where :math:`\\xi \\in [x_0,x_N]` and :math:`\\Delta x = \\frac{x_N-x_0}{N}`
+    is the averages samples spacing.
 
-     If the samples are equally-spaced and N is even, then the error
-     term is $B_N (\Delta x)^{N+3} f^{N+2}(\xi)$.
+    If the samples are equally-spaced and N is even, then the error
+    term is :math:`B_N (\\Delta x)^{N+3} f^{N+2}(\\xi)`.
 
-     Normally, the Newton-Cotes rules are used on smaller integration
-     regions and a composite rule is used to return the total integral.
+    Parameters
+    ----------
 
-    Inputs:
-        rn    -- the integer order for equally-spaced data
-                 or the relative positions of the samples with
-                 the first sample at 0 and the last at N, where
-                 N+1 is the length of rn.  N is the order of the Newt
-        equal -- Set to 1 to enforce equally spaced data
+    rn : int
+        The integer order for equally-spaced data
+        or the relative positions of the samples with
+        the first sample at 0 and the last at N, where
+        N+1 is the length of rn.  N is the order of the Newton
+    equal: int, optional
+        Set to 1 to enforce equally spaced data
+
+    Returns
+    -------
+    an : array
+        1-d array of weights to apply to the function at
+        the provided sample positions.
+    B  : float
+        error coefficient
+
+    Notes
+    -----
+    Normally, the Newton-Cotes rules are used on smaller integration
+    regions and a composite rule is used to return the total integral.
 
-    Outputs:
-        an    -- 1-d array of weights to apply to the function at
-                 the provided sample positions.
-        B     -- error coefficient
     """
     try:
         N = len(rn)-1
@@ -562,7 +723,7 @@ def newton_cotes(rn,equal=0):
         rn = np.arange(N+1)
         equal = 1
 
-    if equal and _builtincoeffs.has_key(N):
+    if equal and N in _builtincoeffs:
         na, da, vi, nb, db = _builtincoeffs[N]
         return na*np.array(vi,float)/da, float(nb)/db
 
@@ -574,7 +735,7 @@ def newton_cotes(rn,equal=0):
     nvec = np.arange(0,N+1)
     C = np.mat(ti**nvec[:,np.newaxis])
     Cinv = C.I
-    # improve precision of result  
+    # improve precision of result
     Cinv = 2*Cinv - Cinv*C*Cinv
     Cinv = 2*Cinv - Cinv*C*Cinv
     Cinv = Cinv.A
@@ -597,4 +758,4 @@ def newton_cotes(rn,equal=0):
 
 # Should only use if samples are forced on you
 def composite(f,x=None,dx=1,axis=-1,n=5):
-    pass 
+    pass
