@@ -52,16 +52,61 @@ if iopt=-1:
                     }
 
 class UnivariateSpline(object):
-    """ Univariate spline s(x) of degree k on the interval
-    [xb,xe] calculated from a given set of data points
-    (x,y).
+    """
+    One-dimensional smoothing spline fit to a given set of data points.
 
-    Can include least-squares fitting.
+    Fits a spline y=s(x) of degree `k` to the provided `x`,`y` data.  `s`
+    specifies the number of knots by specifying a smoothing condition.
 
-    See also:
+    Parameters
+    ----------
+    x : sequence
+        input dimension of data points -- must be increasing
+    y : sequence
+         input dimension of data points
+    w : sequence or None, optional
+        weights for spline fitting.  Must be positive.  If None (default),
+        weights are all equal.
+    bbox : sequence or None, optional
+        2-sequence specifying the boundary of the approximation interval. If
+        None (default), bbox=[x[0],x[-1]].
+    k : int, optional
+        Degree of the smoothing spline.  Must be <= 5.
+    s : float or None, optional
+        Positive smoothing factor used to choose the number of knots.  Number
+        of knots will be increased until the smoothing condition is satisfied:
+
+        sum((w[i]*(y[i]-s(x[i])))**2,axis=0) <= s
+
+        If None (default), s=len(w) which should be a good value if 1/w[i] is
+        an estimate of the standard deviation of y[i].  If 0, spline will
+        interpolate through all data points.
+
+
+    See Also
+    --------
+    InterpolatedUnivariateSpline : Subclass with smoothing forced to 0
+    LSQUnivariateSpline : Subclass in which knots are user-selected instead of
+        being set by smoothing condition
+    splrep : An older, non object-oriented wrapping of FITPACK
+    splev, sproot, splint, spalde
+    BivariateSpline : A similar class for two-dimensional spline interpolation
+
+
+
+    Examples
+    --------
+    >>> from numpy import linspace,exp
+    >>> from numpy.random import randn
+    >>> from scipy.interpolate import UnivariateSpline
+    >>> x = linspace(-3,3,100)
+    >>> y = exp(-x**2) + randn(100)/10
+    >>> s = UnivariateSpline(x,y,s=1)
+    >>> xs = linspace(-3,3,1000)
+    >>> ys = s(xs)
+
+    xs,ys is now a smoothed, super-sampled version of the noisy gaussian x,y
 
-    splrep, splev, sproot, spint, spalde - an older wrapping of FITPACK
-    BivariateSpline - a similar class for bivariate spline interpolation
     """
 
     def __init__(self, x, y, w=None, bbox = [None]*2, k=3, s=None):
@@ -103,19 +148,28 @@ class UnivariateSpline(object):
             pass
         elif ier==-1:
             # the spline returned is an interpolating spline
-            self.__class__ = InterpolatedUnivariateSpline
+            self._set_class(InterpolatedUnivariateSpline)
         elif ier==-2:
             # the spline returned is the weighted least-squares
             # polynomial of degree k. In this extreme case fp gives
             # the upper bound fp0 for the smoothing factor s.
-            self.__class__ = LSQUnivariateSpline
+            self._set_class(LSQUnivariateSpline)
         else:
             # error
             if ier==1:
-                self.__class__ = LSQUnivariateSpline
+                self._set_class(LSQUnivariateSpline)
             message = _curfit_messages.get(ier,'ier=%s' % (ier))
             warnings.warn(message)
 
+    def _set_class(self, cls):
+        self._spline_class = cls
+        if self.__class__ in (UnivariateSpline, InterpolatedUnivariateSpline,
+                              LSQUnivariateSpline):
+            self.__class__ = cls
+        else:
+            # It's an unknown subclass -- don't change class. cf. #731
+            pass
+
     def _reset_nest(self, data, nest=None):
         n = data[10]
         if nest is None:
@@ -136,7 +190,7 @@ class UnivariateSpline(object):
     def set_smoothing_factor(self, s):
         """ Continue spline computation with the given smoothing
         factor s and with the knots found at the last call.
-        
+
         """
         data = self._data
         if data[6]==-1:
@@ -155,7 +209,7 @@ class UnivariateSpline(object):
         """ Evaluate spline (or its nu-th derivative) at positions x.
         Note: x can be unordered but the evaluation is more efficient
         if x is (partially) ordered.
-        
+
         """
         if nu is None:
             return dfitpack.splev(*(self._eval_args+(x,)))
@@ -178,7 +232,7 @@ class UnivariateSpline(object):
     def get_residual(self):
         """Return weighted sum of squared residuals of the spline
         approximation: sum ((w[i]*(y[i]-s(x[i])))**2,axis=0)
-        
+
         """
         return self._data[10]
 
@@ -208,8 +262,52 @@ class UnivariateSpline(object):
               'finding roots unsupported for non-cubic splines'
 
 class InterpolatedUnivariateSpline(UnivariateSpline):
-    """ Interpolated univariate spline approximation. Identical to
-    UnivariateSpline with less error checking.
+    """
+    One-dimensional interpolating spline for a given set of data points.
+
+    Fits a spline y=s(x) of degree `k` to the provided `x`,`y` data. Spline
+    function passes through all provided points. Equivalent to
+    `UnivariateSpline` with  s=0.
+
+    Parameters
+    ----------
+    x : sequence
+        input dimension of data points -- must be increasing
+    y : sequence
+         input dimension of data points
+    w : sequence or None, optional
+        weights for spline fitting.  Must be positive.  If None (default),
+        weights are all equal.
+    bbox : sequence or None, optional
+        2-sequence specifying the boundary of the approximation interval. If
+        None (default), bbox=[x[0],x[-1]].
+    k : int, optional
+        Degree of the smoothing spline.  Must be <= 5.
+
+
+    See Also
+    --------
+    UnivariateSpline : Superclass -- allows knots to be selected by a
+        smoothing condition
+    LSQUnivariateSpline : spline for which knots are user-selected
+    splrep : An older, non object-oriented wrapping of FITPACK
+    splev, sproot, splint, spalde
+    BivariateSpline : A similar class for two-dimensional spline interpolation
+
+
+
+    Examples
+    --------
+    >>> from numpy import linspace,exp
+    >>> from numpy.random import randn
+    >>> from scipy.interpolate import UnivariateSpline
+    >>> x = linspace(-3,3,100)
+    >>> y = exp(-x**2) + randn(100)/10
+    >>> s = UnivariateSpline(x,y,s=1)
+    >>> xs = linspace(-3,3,1000)
+    >>> ys = s(xs)
+
+    xs,ys is now a smoothed, super-sampled version of the noisy gaussian x,y
 
     """
 
@@ -232,9 +330,61 @@ class InterpolatedUnivariateSpline(UnivariateSpline):
         self._reset_class()
 
 class LSQUnivariateSpline(UnivariateSpline):
-    """ Weighted least-squares univariate spline
-    approximation. Appears to be identical to UnivariateSpline with
-    more error checking.
+    """
+    One-dimensional spline with explicit internal knots.
+
+    Fits a spline y=s(x) of degree `k` to the provided `x`,`y` data.  `t`
+    specifies the internal knots of the spline
+
+    Parameters
+    ----------
+    x : sequence
+        input dimension of data points -- must be increasing
+    y : sequence
+        input dimension of data points
+    t: sequence
+        interior knots of the spline.  Must be in ascending order
+        and bbox[0]<t[0]<...<t[-1]<bbox[-1]
+    w : sequence or None, optional
+        weights for spline fitting.  Must be positive.  If None (default),
+        weights are all equal.
+    bbox : sequence or None, optional
+        2-sequence specifying the boundary of the approximation interval. If
+        None (default), bbox=[x[0],x[-1]].
+    k : int, optional
+        Degree of the smoothing spline.  Must be <= 5.
+
+    Raises
+    ------
+
+    ValueError
+        If the interior knots do not satisfy the Schoenberg-Whitney conditions
+
+    See Also
+    --------
+    UnivariateSpline : Superclass -- knots are specified by setting a
+        smoothing condition
+    InterpolatedUnivariateSpline : spline passing through all points
+    splrep : An older, non object-oriented wrapping of FITPACK
+    splev, sproot, splint, spalde
+    BivariateSpline : A similar class for two-dimensional spline interpolation
+
+
+
+    Examples
+    --------
+    >>> from numpy import linspace,exp
+    >>> from numpy.random import randn
+    >>> from scipy.interpolate import LSQUnivariateSpline
+    >>> x = linspace(-3,3,100)
+    >>> y = exp(-x**2) + randn(100)/10
+    >>> t = [-1,0,1]
+    >>> s = LSQUnivariateSpline(x,y,t)
+    >>> xs = linspace(-3,3,1000)
+    >>> ys = s(xs)
+
+    xs,ys is now a smoothed, super-sampled version of the noisy gaussian x,y
+    with knots [-3,-1,0,1,3]
 
     """
 
@@ -322,7 +472,7 @@ class BivariateSpline(object):
 
     bisplrep, bisplev - an older wrapping of FITPACK
     UnivariateSpline - a similar class for univariate spline interpolation
-    SmoothUnivariateSpline - to create a BivariateSpline through the 
+    SmoothUnivariateSpline - to create a BivariateSpline through the
                              given points
     LSQUnivariateSpline - to create a BivariateSpline using weighted
                           least-squares fitting
@@ -353,6 +503,37 @@ class BivariateSpline(object):
             return z
         raise NotImplementedError
 
+    def ev(self, xi, yi):
+        """
+        Evaluate spline at points (x[i], y[i]), i=0,...,len(x)-1
+        """
+        tx,ty,c = self.tck[:3]
+        kx,ky = self.degrees
+        zi,ier = dfitpack.bispeu(tx,ty,c,kx,ky,xi,yi)
+        assert ier==0, 'Invalid input: ier='+`ier`
+        return zi
+
+    def integral(self, xa, xb, ya, yb):
+        """
+        Evaluate the integral of the spline over area [xa,xb] x [ya,yb].
+
+        Parameters
+        ----------
+        xa, xb : float
+            The end-points of the x integration interval.
+        ya, yb : float
+            The end-points of the y integration interval.
+
+        Returns
+        -------
+        integ : float
+            The value of the resulting integral.
+
+        """
+        tx,ty,c = self.tck[:3]
+        kx,ky = self.degrees
+        return dfitpack.dblint(tx,ty,c,kx,ky,xa,xb,ya,yb)
+
 class SmoothBivariateSpline(BivariateSpline):
     """ Smooth bivariate spline approximation.
 
@@ -408,7 +589,7 @@ class LSQBivariateSpline(BivariateSpline):
 
     bisplrep, bisplev - an older wrapping of FITPACK
     UnivariateSpline - a similar class for univariate spline interpolation
-    SmoothUnivariateSpline - to create a BivariateSpline through the 
+    SmoothUnivariateSpline - to create a BivariateSpline through the
                              given points
     """
 
@@ -495,9 +676,9 @@ class RectBivariateSpline(BivariateSpline):
         if not all(diff(y) > 0.0):
             raise TypeError,'y must be strictly increasing'
         if not ((x.min() == x[0]) and (x.max() == x[-1])):
-            raise TypeError, 'x must be strictly ascending' 
+            raise TypeError, 'x must be strictly ascending'
         if not ((y.min() == y[0]) and (y.max() == y[-1])):
-            raise TypeError, 'y must be strictly ascending' 
+            raise TypeError, 'y must be strictly ascending'
         if not x.size == z.shape[0]:
             raise TypeError,\
                   'x dimension of z must have same number of elements as x'
@@ -518,4 +699,3 @@ class RectBivariateSpline(BivariateSpline):
         self.fp = fp
         self.tck = tx[:nx],ty[:ny],c[:(nx-kx-1)*(ny-ky-1)]
         self.degrees = kx,ky
-
