@@ -4,10 +4,16 @@
 Need function load / save / roundtrip tests
 
 '''
-from os.path import join, dirname
+from os.path import join as pjoin, dirname
 from glob import glob
 from StringIO import StringIO
 from tempfile import mkdtemp
+# functools is only available in Python >=2.5
+try:
+    from functools import partial
+except ImportError:
+    from scipy.io.arff.myfunctools import partial
+
 import warnings
 import shutil
 import gzip
@@ -24,13 +30,25 @@ import numpy as np
 from numpy import array
 import scipy.sparse as SP
 
-from scipy.io.matlab.miobase import matdims
-from scipy.io.matlab.mio import loadmat, savemat, find_mat_file, \
-     mat_reader_factory
+import scipy.io.matlab.byteordercodes as boc
+from scipy.io.matlab.miobase import matdims, MatFileReader, \
+    MatWriteError
+from scipy.io.matlab.mio import find_mat_file, mat_reader_factory, \
+    loadmat, savemat
 from scipy.io.matlab.mio5 import MatlabObject, MatFile5Writer, \
-     Mat5NumericWriter
+      MatFile5Reader, MatlabFunction
 
-test_data_path = join(dirname(__file__), 'data')
+# Use future defaults to silence unwanted test warnings
+savemat_future = partial(savemat, oned_as='row')
+class MatFile5Reader_future(MatFile5Reader):
+    def __init__(self, *args, **kwargs):
+        sar = kwargs.get('struct_as_record')
+        if sar is None:
+            kwargs['struct_as_record'] = True
+        super(MatFile5Reader_future, self).__init__(*args, **kwargs)
+
+
+test_data_path = pjoin(dirname(__file__), 'data')
 
 def mlarr(*args, **kwargs):
     ''' Convenience function to return matlab-compatible 2D array
@@ -175,7 +193,7 @@ case_table5.append(
      'expected': {'testobject': MO}
      })
 u_str = file(
-    join(test_data_path, 'japanese_utf8.txt'),
+    pjoin(test_data_path, 'japanese_utf8.txt'),
     'rb').read().decode('utf-8')
 case_table5.append(
     {'name': 'unicode',
@@ -189,13 +207,8 @@ case_table5.append(
     {'name': 'sparsecomplex',
      'expected': {'testsparsecomplex': SP.coo_matrix(B)},
      })
-# We cannot read matlab functions for the moment
-case_table5.append(
-    {'name': 'func',
-     'expected': {'testfunc': 'Read error: Cannot read matlab functions'},
-     })
 
-case_table5_rt = case_table5[:-1] # not the function read write
+case_table5_rt = case_table5[:]
 # Inline functions can't be concatenated in matlab, so RT only
 case_table5_rt.append(
     {'name': 'objectarray',
@@ -204,7 +217,7 @@ case_table5_rt.append(
 
 def types_compatible(var1, var2):
     ''' Check if types are same or compatible
-    
+
     0d numpy scalars are compatible with bare python scalars
     '''
     type1 = type(var1)
@@ -229,7 +242,7 @@ def _check_level(label, expected, actual):
         return
     # Check types are as expected
     assert_true(types_compatible(expected, actual), \
-           "Expected type %s, got %s at %s" % 
+           "Expected type %s, got %s at %s" %
                 (type(expected), type(actual), label))
     # A field in a record array may not be an ndarray
     # A scalar from a record array will be type np.void
@@ -278,7 +291,7 @@ def _load_check_case(name, files, case):
 # Round trip tests
 def _rt_check_case(name, expected, format):
     mat_stream = StringIO()
-    savemat(mat_stream, expected, format=format)
+    savemat_future(mat_stream, expected, format=format)
     mat_stream.seek(0)
     _load_check_case(name, [mat_stream], expected)
 
@@ -288,7 +301,7 @@ def test_load():
     for case in case_table4 + case_table5:
         name = case['name']
         expected = case['expected']
-        filt = join(test_data_path, 'test%s_*.mat' % name)
+        filt = pjoin(test_data_path, 'test%s_*.mat' % name)
         files = glob(filt)
         assert_true(len(files) > 0,
                     "No files for test %s using filter %s" % (name, filt))
@@ -316,9 +329,9 @@ def test_gzip_simple():
 
     tmpdir = mkdtemp()
     try:
-        fname = join(tmpdir,name)
+        fname = pjoin(tmpdir,name)
         mat_stream = gzip.open( fname,mode='wb')
-        savemat(mat_stream, expected, format=format)
+        savemat_future(mat_stream, expected, format=format)
         mat_stream.close()
 
         mat_stream = gzip.open( fname,mode='rb')
@@ -334,7 +347,7 @@ def test_gzip_simple():
 def test_mat73():
     # Check any hdf5 files raise an error
     filenames = glob(
-        join(test_data_path, 'testhdf5*.mat'))
+        pjoin(test_data_path, 'testhdf5*.mat'))
     assert_true(len(filenames)>0)
     for filename in filenames:
         assert_raises(NotImplementedError,
@@ -344,26 +357,20 @@ def test_mat73():
 
 
 def test_warnings():
-    fname = join(test_data_path, 'testdouble_7.1_GLNX86.mat')
+    fname = pjoin(test_data_path, 'testdouble_7.1_GLNX86.mat')
     warnings.simplefilter('error')
     # This should not generate a warning
     mres = loadmat(fname, struct_as_record=True)
     # This neither
     mres = loadmat(fname, struct_as_record=False)
-    # This should
-    yield assert_raises, FutureWarning, loadmat, fname
-    # This too
-    yield assert_raises, FutureWarning, find_mat_file, fname
-    # we need kwargs for this one
-    yield (lambda a, k: assert_raises(*a, **k),
-          (DeprecationWarning, loadmat, fname),
-          {'struct_as_record':True, 'basename':'raw'})
+    # This should - because of deprecated system path search
+    yield assert_raises, DeprecationWarning, find_mat_file, fname
     warnings.resetwarnings()
 
 
 def test_regression_653():
     """Regression test for #653."""
-    assert_raises(TypeError, savemat, StringIO(), {'d':{1:2}}, format='5')
+    assert_raises(TypeError, savemat_future, StringIO(), {'d':{1:2}}, format='5')
 
 
 def test_structname_len():
@@ -372,17 +379,17 @@ def test_structname_len():
     fldname = 'a' * lim
     st1 = np.zeros((1,1), dtype=[(fldname, object)])
     mat_stream = StringIO()
-    savemat(StringIO(), {'longstruct': st1}, format='5')
+    savemat_future(StringIO(), {'longstruct': st1}, format='5')
     fldname = 'a' * (lim+1)
     st1 = np.zeros((1,1), dtype=[(fldname, object)])
-    assert_raises(ValueError, savemat, StringIO(),
+    assert_raises(ValueError, savemat_future, StringIO(),
                   {'longstruct': st1}, format='5')
 
 
 def test_4_and_long_field_names_incompatible():
     # Long field names option not supported in 4
     my_struct = np.zeros((1,1),dtype=[('my_fieldname',object)])
-    assert_raises(ValueError, savemat, StringIO(),
+    assert_raises(ValueError, savemat_future, StringIO(),
                   {'my_struct':my_struct}, format='4', long_field_names=True)
 
 
@@ -392,10 +399,10 @@ def test_long_field_names():
     fldname = 'a' * lim
     st1 = np.zeros((1,1), dtype=[(fldname, object)])
     mat_stream = StringIO()
-    savemat(StringIO(), {'longstruct': st1}, format='5',long_field_names=True)
+    savemat_future(StringIO(), {'longstruct': st1}, format='5',long_field_names=True)
     fldname = 'a' * (lim+1)
     st1 = np.zeros((1,1), dtype=[(fldname, object)])
-    assert_raises(ValueError, savemat, StringIO(),
+    assert_raises(ValueError, savemat_future, StringIO(),
                   {'longstruct': st1}, format='5',long_field_names=True)
 
 
@@ -409,11 +416,11 @@ def test_long_field_names_in_struct():
     cell[0,0]=st1
     cell[0,1]=st1
     mat_stream = StringIO()
-    savemat(StringIO(), {'longstruct': cell}, format='5',long_field_names=True)
+    savemat_future(StringIO(), {'longstruct': cell}, format='5',long_field_names=True)
     #
     # Check to make sure it fails with long field names off
     #
-    assert_raises(ValueError, savemat, StringIO(),
+    assert_raises(ValueError, savemat_future, StringIO(),
                   {'longstruct': cell}, format='5', long_field_names=False)
 
 
@@ -425,17 +432,17 @@ def test_cell_with_one_thing_in_it():
     cells[0,0]='Hello'
     cells[0,1]='World'
     mat_stream = StringIO()
-    savemat(StringIO(), {'x': cells}, format='5')
+    savemat_future(StringIO(), {'x': cells}, format='5')
 
     cells = np.ndarray((1,1),dtype=object)
     cells[0,0]='Hello, world'
     mat_stream = StringIO()
-    savemat(StringIO(), {'x': cells}, format='5')
+    savemat_future(StringIO(), {'x': cells}, format='5')
 
 
 def test_writer_properties():
     # Tests getting, setting of properties of matrix writer
-    mfw = MatFile5Writer(StringIO())
+    mfw = MatFile5Writer(StringIO(), oned_as='row')
     yield assert_equal, mfw.global_vars, []
     mfw.global_vars = ['avar']
     yield assert_equal, mfw.global_vars, ['avar']
@@ -450,16 +457,18 @@ def test_writer_properties():
 def test_use_small_element():
     # Test whether we're using small data element or not
     sio = StringIO()
+    wtr = MatFile5Writer(sio, oned_as='column')
     # First check size for no sde for name
-    writer = Mat5NumericWriter(sio, np.zeros(10), 'aaaaa').write()
+    arr = np.zeros(10)
+    wtr.put_variables({'aaaaa': arr})
     w_sz = sio.len
     # Check small name results in largish difference in size
     sio.truncate(0)
-    writer = Mat5NumericWriter(sio, np.zeros(10), 'aaaa').write()
+    wtr.put_variables({'aaaa': arr})
     yield assert_true, w_sz - sio.len > 4
     # Whereas increasing name size makes less difference
     sio.truncate(0)
-    writer = Mat5NumericWriter(sio, np.zeros(10), 'aaaaaa').write()
+    wtr.put_variables({'aaaaaa': arr})
     yield assert_true, sio.len - w_sz < 4
 
 
@@ -467,7 +476,7 @@ def test_save_dict():
     # Test that dict can be saved (as recarray), loaded as matstruct
     d = {'a':1, 'b':2}
     stream = StringIO()
-    savemat(stream, {'dict':d})
+    savemat_future(stream, {'dict':d})
     stream.seek(0)
     vals = loadmat(stream)
 
@@ -476,11 +485,12 @@ def test_1d_shape():
     # Current 5 behavior is 1D -> column vector
     arr = np.arange(5)
     stream = StringIO()
+    # silence warnings for tests
+    warnings.simplefilter('ignore')
     savemat(stream, {'oned':arr}, format='5')
     vals = loadmat(stream)
     yield assert_equal, vals['oned'].shape, (5,1)
     # Current 4 behavior is 1D -> row vector
-    arr = np.arange(5)
     stream = StringIO()
     savemat(stream, {'oned':arr}, format='4')
     vals = loadmat(stream)
@@ -488,30 +498,31 @@ def test_1d_shape():
     for format in ('4', '5'):
         # can be explicitly 'column' for oned_as
         stream = StringIO()
-        savemat(stream, {'oned':arr}, 
+        savemat(stream, {'oned':arr},
                 format=format,
                 oned_as='column')
         vals = loadmat(stream)
         yield assert_equal, vals['oned'].shape, (5,1)
         # but different from 'row'
         stream = StringIO()
-        savemat(stream, {'oned':arr}, 
+        savemat(stream, {'oned':arr},
                 format=format,
                 oned_as='row')
         vals = loadmat(stream)
         yield assert_equal, vals['oned'].shape, (1,5)
-    
+    warnings.resetwarnings()
+
 
 def test_compression():
     arr = np.zeros(100).reshape((5,20))
     arr[2,10] = 1
     stream = StringIO()
-    savemat(stream, {'arr':arr})
+    savemat_future(stream, {'arr':arr})
     raw_len = len(stream.getvalue())
     vals = loadmat(stream)
     yield assert_array_equal, vals['arr'], arr
     stream = StringIO()
-    savemat(stream, {'arr':arr}, do_compression=True)
+    savemat_future(stream, {'arr':arr}, do_compression=True)
     compressed_len = len(stream.getvalue())
     vals = loadmat(stream)
     yield assert_array_equal, vals['arr'], arr
@@ -520,18 +531,19 @@ def test_compression():
     arr2 = arr.copy()
     arr2[0,0] = 1
     stream = StringIO()
-    savemat(stream, {'arr':arr, 'arr2':arr2}, do_compression=False)
+    savemat_future(stream, {'arr':arr, 'arr2':arr2}, do_compression=False)
     vals = loadmat(stream)
     yield assert_array_equal, vals['arr2'], arr2
     stream = StringIO()
-    savemat(stream, {'arr':arr, 'arr2':arr2}, do_compression=True)
+    savemat_future(stream, {'arr':arr, 'arr2':arr2}, do_compression=True)
     vals = loadmat(stream)
     yield assert_array_equal, vals['arr2'], arr2
-    
+
 
 def test_single_object():
     stream = StringIO()
-    savemat(stream, {'A':np.array(1, dtype=object)})
+    savemat_future(stream, {'A':np.array(1, dtype=object)})
+
 
 def test_skip_variable():
     # Test skipping over the first of two variables in a MAT file
@@ -544,7 +556,7 @@ def test_skip_variable():
     # The problem arises when the chunk is large: this file has
     # a 256x256 array of random (uncompressible) doubles.
     #
-    filename = join(test_data_path,'test_skip_variable.mat')
+    filename = pjoin(test_data_path,'test_skip_variable.mat')
     #
     # Prove that it loads with loadmat
     #
@@ -564,7 +576,7 @@ def test_skip_variable():
 
 def test_empty_struct():
     # ticket 885
-    filename = join(test_data_path,'test_empty_struct.mat')
+    filename = pjoin(test_data_path,'test_empty_struct.mat')
     # before ticket fix, this would crash with ValueError, empty data
     # type
     d = loadmat(filename, struct_as_record=True)
@@ -575,7 +587,7 @@ def test_empty_struct():
     stream = StringIO()
     arr = np.array((), dtype='U')
     # before ticket fix, this used to give data type not understood
-    savemat(stream, {'arr':arr})
+    savemat_future(stream, {'arr':arr})
     d = loadmat(stream)
     a2 = d['arr']
     yield assert_array_equal, a2, arr
@@ -591,7 +603,7 @@ def test_recarray():
     arr[1]['f1'] = 99
     arr[1]['f2'] = 'not perl'
     stream = StringIO()
-    savemat(stream, {'arr': arr})
+    savemat_future(stream, {'arr': arr})
     d = loadmat(stream, struct_as_record=False)
     a20 = d['arr'][0,0]
     yield assert_equal, a20.f1, 0.5
@@ -606,7 +618,7 @@ def test_recarray():
     a21 = d['arr'].flat[1]
     yield assert_equal, a21['f1'], 99
     yield assert_equal, a21['f2'], 'not perl'
-          
+
 
 def test_save_object():
     class C(object): pass
@@ -614,7 +626,7 @@ def test_save_object():
     c.field1 = 1
     c.field2 = 'a string'
     stream = StringIO()
-    savemat(stream, {'c': c})
+    savemat_future(stream, {'c': c})
     d = loadmat(stream, struct_as_record=False)
     c2 = d['c'][0,0]
     yield assert_equal, c2.field1, 1
@@ -623,3 +635,147 @@ def test_save_object():
     c2 = d['c'][0,0]
     yield assert_equal, c2['field1'], 1
     yield assert_equal, c2['field2'], 'a string'
+
+
+def test_read_opts():
+    # tests if read is seeing option sets, at initialization and after
+    # initialization
+    arr = np.arange(6).reshape(1,6)
+    stream = StringIO()
+    savemat_future(stream, {'a': arr})
+    rdr = MatFile5Reader_future(stream)
+    back_dict = rdr.get_variables()
+    rarr = back_dict['a']
+    yield assert_array_equal, rarr, arr
+    rdr = MatFile5Reader_future(stream, squeeze_me=True)
+    yield assert_array_equal, rdr.get_variables()['a'], arr.reshape((6,))
+    rdr.squeeze_me = False
+    yield assert_array_equal, rarr, arr
+    rdr = MatFile5Reader_future(stream, byte_order=boc.native_code)
+    yield assert_array_equal, rdr.get_variables()['a'], arr
+    # inverted byte code leads to error on read because of swapped
+    # header etc
+    rdr = MatFile5Reader_future(stream, byte_order=boc.swapped_code)
+    yield assert_raises, Exception, rdr.get_variables
+    rdr.byte_order = boc.native_code
+    yield assert_array_equal, rdr.get_variables()['a'], arr
+    arr = np.array(['a string'])
+    stream.truncate(0)
+    savemat_future(stream, {'a': arr})
+    rdr = MatFile5Reader_future(stream)
+    yield assert_array_equal, rdr.get_variables()['a'], arr
+    rdr = MatFile5Reader_future(stream, chars_as_strings=False)
+    carr = np.atleast_2d(np.array(list(arr.item()), dtype='U1'))
+    yield assert_array_equal, rdr.get_variables()['a'], carr
+    rdr.chars_as_strings=True
+    yield assert_array_equal, rdr.get_variables()['a'], arr
+
+
+def test_empty_string():
+    # make sure reading empty string does not raise error
+    estring_fname = pjoin(test_data_path, 'single_empty_string.mat')
+    rdr = MatFile5Reader_future(file(estring_fname, 'rb'))
+    d = rdr.get_variables()
+    yield assert_array_equal, d['a'], np.array([], dtype='U1')
+    # empty string round trip.  Matlab cannot distiguish
+    # between a string array that is empty, and a string array
+    # containing a single empty string, because it stores strings as
+    # arrays of char.  There is no way of having an array of char that
+    # is not empty, but contains an empty string.
+    stream = StringIO()
+    savemat_future(stream, {'a': np.array([''])})
+    rdr = MatFile5Reader_future(stream)
+    d = rdr.get_variables()
+    yield assert_array_equal, d['a'], np.array([], dtype='U1')
+    stream.truncate(0)
+    savemat_future(stream, {'a': np.array([], dtype='U1')})
+    rdr = MatFile5Reader_future(stream)
+    d = rdr.get_variables()
+    yield assert_array_equal, d['a'], np.array([], dtype='U1')
+
+
+def test_mat4_3d():
+    # test behavior when writing 3D arrays to matlab 4 files
+    stream = StringIO()
+    arr = np.arange(24).reshape((2,3,4))
+    warnings.simplefilter('error')
+    yield (assert_raises, DeprecationWarning, savemat_future,
+           stream, {'a': arr}, True, '4')
+    warnings.resetwarnings()
+    # For now, we save a 3D array as 2D
+    warnings.simplefilter('ignore')
+    savemat_future(stream, {'a': arr}, format='4')
+    warnings.resetwarnings()
+    d = loadmat(stream)
+    yield assert_array_equal, d['a'], arr.reshape((6,4))
+
+
+def test_func_read():
+    func_eg = pjoin(test_data_path, 'testfunc_7.4_GLNX86.mat')
+    rdr = MatFile5Reader_future(file(func_eg, 'rb'))
+    d = rdr.get_variables()
+    yield assert_true, isinstance(d['testfunc'], MatlabFunction)
+    stream = StringIO()
+    wtr = MatFile5Writer(stream, oned_as='row')
+    yield assert_raises, MatWriteError, wtr.put_variables, d
+
+
+def test_mat_dtype():
+    double_eg = pjoin(test_data_path, 'testmatrix_6.1_SOL2.mat')
+    rdr = MatFile5Reader_future(file(double_eg, 'rb'), mat_dtype=False)
+    d = rdr.get_variables()
+    yield assert_equal, d['testmatrix'].dtype.kind, 'u'
+    rdr = MatFile5Reader_future(file(double_eg, 'rb'), mat_dtype=True)
+    d = rdr.get_variables()
+    yield assert_equal, d['testmatrix'].dtype.kind, 'f'
+
+
+def test_sparse_in_struct():
+    # reproduces bug found by DC where Cython code was insisting on
+    # ndarray return type, but getting sparse matrix
+    st = {'sparsefield': SP.coo_matrix(np.eye(4))}
+    stream = StringIO()
+    savemat_future(stream, {'a':st})
+    d = loadmat(stream, struct_as_record=True)
+    yield assert_array_equal, d['a'][0,0]['sparsefield'].todense(), np.eye(4)
+
+
+def test_mat_struct_squeeze():
+    stream = StringIO()
+    in_d = {'st':{'one':1, 'two':2}}
+    savemat_future(stream, in_d)
+    # no error without squeeze
+    out_d = loadmat(stream, struct_as_record=False)
+    # previous error was with squeeze, with mat_struct
+    out_d = loadmat(stream,
+                    struct_as_record=False,
+                    squeeze_me=True,
+                    )
+
+
+def test_str_round():
+    # from report by Angus McMorland on mailing list 3 May 2010
+    stream = StringIO()
+    in_arr = np.array(['Hello', 'Foob'])
+    out_arr = np.array(['Hello', 'Foob '])
+    savemat_future(stream, dict(a=in_arr))
+    res = loadmat(stream)
+    # resulted in [u'HloolFoa', u'elWrdobr']
+    yield assert_array_equal, res['a'], out_arr
+    stream.truncate(0)
+    # Make Fortran ordered version of string
+    in_str = in_arr.tostring(order='F')
+    in_from_str = np.ndarray(shape=a.shape,
+                             dtype=in_arr.dtype,
+                             order='F',
+                             buffer=in_str)
+    savemat_future(stream, dict(a=in_from_str))
+    yield assert_array_equal, res['a'], out_arr
+    # unicode save did lead to buffer too small error
+    stream.truncate(0)
+    in_arr_u = in_arr.astype('U')
+    out_arr_u = out_arr.astype('U')
+    savemat_future(stream, {'a': in_arr_u})
+    res = loadmat(stream)
+    yield assert_array_equal, res['a'], out_arr_u
+
