@@ -19,37 +19,247 @@ Run tests if linalg is not installed:
   python tests/test_basic.py
 """
 
-from numpy import arange, add, array, dot, zeros, identity, conjugate, transpose
+import warnings
+
+from numpy import arange, array, dot, zeros, identity, conjugate, transpose, \
+        float32, zeros_like
 import numpy.linalg as linalg
 
 from numpy.testing import *
 
-from scipy.linalg import solve,inv,det,lstsq, toeplitz, hankel, tri, triu, \
-     tril, pinv, pinv2, solve_banded
+from scipy.linalg import solve, inv, det, lstsq, pinv, pinv2, norm,\
+        solve_banded, solveh_banded, cholesky_banded
 
 
 def random(size):
     return rand(*size)
 
-def get_mat(n):
-    data = arange(n)
-    data = add.outer(data,data)
-    return data
 
 class TestSolveBanded(TestCase):
 
-    def test_simple(self):
-
-        a = [[1,20,0,0],[-30,4,6,0],[2,1,20,2],[0,-1,7,14]]
-        ab = [[0,20,6,2],
-              [1,4,20,14],
-              [-30,1,7,0],
-              [2,-1,0,0]]
+    def test_real(self):
+        a = array([[ 1.0, 20,  0,  0],
+                   [ -30,  4,  6,  0],
+                   [   2,  1, 20,  2],
+                   [   0, -1,  7, 14]])
+        ab = array([[ 0.0, 20,  6,  2],
+                    [   1,  4, 20, 14],
+                    [ -30,  1,  7,  0],
+                    [   2, -1,  0,  0]])
         l,u = 2,1
-        for b in ([[1,0,0,0],[0,0,0,1],[0,1,0,0],[0,1,0,0]],
-                  [[2,1],[-30,4],[2,3],[1,3]]):
-            x = solve_banded((l,u),ab,b)
-            assert_array_almost_equal(dot(a,x),b)
+        b4 = array([10.0, 0.0, 2.0, 14.0])
+        b4by1 = b4.reshape(-1,1)
+        b4by2 = array([[ 2, 1],
+                       [-30, 4],
+                       [  2, 3],
+                       [  1, 3]])
+        b4by4 = array([[1, 0, 0, 0],
+                       [0, 0, 0, 1],
+                       [0, 1, 0, 0],
+                       [0, 1, 0, 0]])
+        for b in [b4, b4by1, b4by2, b4by4]:
+            x = solve_banded((l, u), ab, b)
+            assert_array_almost_equal(dot(a, x), b)
+
+    def test_complex(self):
+        a = array([[ 1.0, 20,  0,   0],
+                   [ -30,  4,  6,   0],
+                   [  2j,  1, 20,  2j],
+                   [   0, -1,  7,  14]])
+        ab = array([[ 0.0, 20,  6,  2j],
+                    [   1,  4, 20,  14],
+                    [ -30,  1,  7,   0],
+                    [  2j, -1,  0,   0]])
+        l,u = 2,1
+        b4 = array([10.0, 0.0, 2.0, 14.0j])
+        b4by1 = b4.reshape(-1,1)
+        b4by2 = array([[ 2, 1],
+                       [-30, 4],
+                       [  2, 3],
+                       [  1, 3]])
+        b4by4 = array([[1, 0, 0, 0],
+                       [0, 0, 0,1j],
+                       [0, 1, 0, 0],
+                       [0, 1, 0, 0]])
+        for b in [b4, b4by1, b4by2, b4by4]:
+            x = solve_banded((l, u), ab, b)
+            assert_array_almost_equal(dot(a, x), b)
+
+    def test_bad_shape(self):
+        ab = array([[ 0.0, 20,  6,  2],
+                    [   1,  4, 20, 14],
+                    [ -30,  1,  7,  0],
+                    [   2, -1,  0,  0]])
+        l,u = 2,1
+        bad = array([1.0, 2.0, 3.0, 4.0]).reshape(-1,4)
+        assert_raises(ValueError, solve_banded, (l, u), ab, bad)
+        assert_raises(ValueError, solve_banded, (l, u), ab, [1.0, 2.0])
+
+        # Values of (l,u) are not compatible with ab.
+        assert_raises(ValueError, solve_banded, (1, 1), ab, [1.0, 2.0])
+
+
+class TestSolveHBanded(TestCase):
+    # solveh_banded currently has a DeprecationWarning.  When the warning
+    # is removed in scipy 0.9, the 'ignore' filters and the test for the
+    # warning can be removed.
+
+    def test_01_upper(self):
+        warnings.simplefilter('ignore', category=DeprecationWarning)
+        # Solve
+        # [ 4 1 0]     [1]
+        # [ 1 4 1] X = [4]
+        # [ 0 1 4]     [1]
+        # with the RHS as a 1D array.
+        ab = array([[-99, 1.0, 1.0], [4.0, 4.0, 4.0]])
+        b = array([1.0, 4.0, 1.0])
+        c, x = solveh_banded(ab, b)
+        assert_array_almost_equal(x, [0.0, 1.0, 0.0])
+        # Remove the following part of this test in scipy 0.9.
+        a = array([[4.0, 1.0, 0.0], [1.0, 4.0, 1.0], [0.0, 1.0, 4.0]])
+        fac = zeros_like(a)
+        fac[range(3),range(3)] = c[-1]
+        fac[(0,1),(1,2)] = c[0,1:]
+        assert_array_almost_equal(a, dot(fac.T, fac))
+
+    def test_02_upper(self):
+        warnings.simplefilter('ignore', category=DeprecationWarning)
+        # Solve
+        # [ 4 1 0]     [1 4]
+        # [ 1 4 1] X = [4 2]
+        # [ 0 1 4]     [1 4]
+        #
+        ab = array([[-99, 1.0, 1.0],
+                    [4.0, 4.0, 4.0]])
+        b = array([[1.0, 4.0],
+                   [4.0, 2.0],
+                   [1.0, 4.0]])
+        c, x = solveh_banded(ab, b)
+        expected = array([[0.0, 1.0],
+                          [1.0, 0.0],
+                          [0.0, 1.0]])
+        assert_array_almost_equal(x, expected)
+
+    def test_03_upper(self):
+        warnings.simplefilter('ignore', category=DeprecationWarning)
+        # Solve
+        # [ 4 1 0]     [1]
+        # [ 1 4 1] X = [4]
+        # [ 0 1 4]     [1]
+        # with the RHS as a 2D array with shape (3,1).
+        ab = array([[-99, 1.0, 1.0], [4.0, 4.0, 4.0]])
+        b = array([1.0, 4.0, 1.0]).reshape(-1,1)
+        c, x = solveh_banded(ab, b)
+        assert_array_almost_equal(x, array([0.0, 1.0, 0.0]).reshape(-1,1))
+
+    def test_01_lower(self):
+        warnings.simplefilter('ignore', category=DeprecationWarning)
+        # Solve
+        # [ 4 1 0]     [1]
+        # [ 1 4 1] X = [4]
+        # [ 0 1 4]     [1]
+        #
+        ab = array([[4.0, 4.0, 4.0],
+                    [1.0, 1.0, -99]])
+        b = array([1.0, 4.0, 1.0])
+        c, x = solveh_banded(ab, b, lower=True)
+        assert_array_almost_equal(x, [0.0, 1.0, 0.0])
+
+    def test_02_lower(self):
+        warnings.simplefilter('ignore', category=DeprecationWarning)
+        # Solve
+        # [ 4 1 0]     [1 4]
+        # [ 1 4 1] X = [4 2]
+        # [ 0 1 4]     [1 4]
+        #
+        ab = array([[4.0, 4.0, 4.0],
+                    [1.0, 1.0, -99]])
+        b = array([[1.0, 4.0],
+                   [4.0, 2.0],
+                   [1.0, 4.0]])
+        c, x = solveh_banded(ab, b, lower=True)
+        expected = array([[0.0, 1.0],
+                          [1.0, 0.0],
+                          [0.0, 1.0]])
+        assert_array_almost_equal(x, expected)
+
+    def test_01_float32(self):
+        warnings.simplefilter('ignore', category=DeprecationWarning)
+        # Solve
+        # [ 4 1 0]     [1]
+        # [ 1 4 1] X = [4]
+        # [ 0 1 4]     [1]
+        #
+        ab = array([[-99, 1.0, 1.0], [4.0, 4.0, 4.0]], dtype=float32)
+        b = array([1.0, 4.0, 1.0], dtype=float32)
+        c, x = solveh_banded(ab, b)
+        assert_array_almost_equal(x, [0.0, 1.0, 0.0])
+
+    def test_02_float32(self):
+        warnings.simplefilter('ignore', category=DeprecationWarning)
+        # Solve
+        # [ 4 1 0]     [1 4]
+        # [ 1 4 1] X = [4 2]
+        # [ 0 1 4]     [1 4]
+        #
+        ab = array([[-99, 1.0, 1.0],
+                    [4.0, 4.0, 4.0]], dtype=float32)
+        b = array([[1.0, 4.0],
+                   [4.0, 2.0],
+                   [1.0, 4.0]], dtype=float32)
+        c, x = solveh_banded(ab, b)
+        expected = array([[0.0, 1.0],
+                          [1.0, 0.0],
+                          [0.0, 1.0]])
+        assert_array_almost_equal(x, expected)
+
+    def test_01_complex(self):
+        warnings.simplefilter('ignore', category=DeprecationWarning)
+        # Solve
+        # [ 4 -j 0]     [ -j]
+        # [ j 4 -j] X = [4-j]
+        # [ 0 j  4]     [4+j]
+        #
+        ab = array([[-99, -1.0j, -1.0j], [4.0, 4.0, 4.0]])
+        b = array([-1.0j, 4.0-1j, 4+1j])
+        c, x = solveh_banded(ab, b)
+        assert_array_almost_equal(x, [0.0, 1.0, 1.0])
+
+    def test_02_complex(self):
+        warnings.simplefilter('ignore', category=DeprecationWarning)
+        # Solve
+        # [ 4 -j 0]     [ -j    4j]
+        # [ j 4 -j] X = [4-j  -1-j]
+        # [ 0 j  4]     [4+j   4  ]
+        #
+        ab = array([[-99, -1.0j, -1.0j],
+                    [4.0, 4.0, 4.0]])
+        b = array([[   -1j,    4.0j],
+                   [4.0-1j, -1.0-1j],
+                   [4.0+1j,     4.0]])
+        c, x = solveh_banded(ab, b)
+        expected = array([[0.0, 1.0j],
+                          [1.0,  0.0],
+                          [1.0,  1.0]])
+        assert_array_almost_equal(x, expected)
+
+    def test_bad_shapes(self):
+        warnings.simplefilter('ignore', category=DeprecationWarning)
+
+        ab = array([[-99, 1.0, 1.0],
+                    [4.0, 4.0, 4.0]])
+        b = array([[1.0, 4.0],
+                   [4.0, 2.0]])
+        assert_raises(ValueError, solveh_banded, ab, b)
+        assert_raises(ValueError, solveh_banded, ab, [1.0, 2.0])
+        assert_raises(ValueError, solveh_banded, ab, [1.0])
+
+    def test_00_deprecation_warning(self):
+        warnings.simplefilter('error', category=DeprecationWarning)
+        ab = array([[-99, 1.0, 1.0], [4.0, 4.0, 4.0]])
+        b = array([1.0, 4.0, 1.0])
+        assert_raises(DeprecationWarning, solveh_banded, ab, b)
+
 
 class TestSolve(TestCase):
 
@@ -304,99 +514,6 @@ class TestLstsq(TestCase):
             #XXX: check definition of res
             assert_array_almost_equal(x,direct_lstsq(a,b,1))
 
-class TestTri(TestCase):
-    def test_basic(self):
-        assert_equal(tri(4),array([[1,0,0,0],
-                                   [1,1,0,0],
-                                   [1,1,1,0],
-                                   [1,1,1,1]]))
-        assert_equal(tri(4,dtype='f'),array([[1,0,0,0],
-                                                [1,1,0,0],
-                                                [1,1,1,0],
-                                                [1,1,1,1]],'f'))
-    def test_diag(self):
-        assert_equal(tri(4,k=1),array([[1,1,0,0],
-                                       [1,1,1,0],
-                                       [1,1,1,1],
-                                       [1,1,1,1]]))
-        assert_equal(tri(4,k=-1),array([[0,0,0,0],
-                                        [1,0,0,0],
-                                        [1,1,0,0],
-                                        [1,1,1,0]]))
-    def test_2d(self):
-        assert_equal(tri(4,3),array([[1,0,0],
-                                     [1,1,0],
-                                     [1,1,1],
-                                     [1,1,1]]))
-        assert_equal(tri(3,4),array([[1,0,0,0],
-                                     [1,1,0,0],
-                                     [1,1,1,0]]))
-    def test_diag2d(self):
-        assert_equal(tri(3,4,k=2),array([[1,1,1,0],
-                                         [1,1,1,1],
-                                         [1,1,1,1]]))
-        assert_equal(tri(4,3,k=-2),array([[0,0,0],
-                                          [0,0,0],
-                                          [1,0,0],
-                                          [1,1,0]]))
-
-class TestTril(TestCase):
-    def test_basic(self):
-        a = (100*get_mat(5)).astype('l')
-        b = a.copy()
-        for k in range(5):
-            for l in range(k+1,5):
-                b[k,l] = 0
-        assert_equal(tril(a),b)
-
-    def test_diag(self):
-        a = (100*get_mat(5)).astype('f')
-        b = a.copy()
-        for k in range(5):
-            for l in range(k+3,5):
-                b[k,l] = 0
-        assert_equal(tril(a,k=2),b)
-        b = a.copy()
-        for k in range(5):
-            for l in range(max((k-1,0)),5):
-                b[k,l] = 0
-        assert_equal(tril(a,k=-2),b)
-
-class TestTriu(TestCase):
-    def test_basic(self):
-        a = (100*get_mat(5)).astype('l')
-        b = a.copy()
-        for k in range(5):
-            for l in range(k+1,5):
-                b[l,k] = 0
-        assert_equal(triu(a),b)
-
-    def test_diag(self):
-        a = (100*get_mat(5)).astype('f')
-        b = a.copy()
-        for k in range(5):
-            for l in range(max((k-1,0)),5):
-                b[l,k] = 0
-        assert_equal(triu(a,k=2),b)
-        b = a.copy()
-        for k in range(5):
-            for l in range(k+3,5):
-                b[l,k] = 0
-        assert_equal(triu(a,k=-2),b)
-
-class TestToeplitz(TestCase):
-    def test_basic(self):
-        y = toeplitz([1,2,3])
-        assert_array_equal(y,[[1,2,3],[2,1,2],[3,2,1]])
-        y = toeplitz([1,2,3],[1,4,5])
-        assert_array_equal(y,[[1,4,5],[2,1,4],[3,2,1]])
-
-class TestHankel(TestCase):
-    def test_basic(self):
-        y = hankel([1,2,3])
-        assert_array_equal(y,[[1,2,3],[2,3,0],[3,0,0]])
-        y = hankel([1,2,3],[3,4,5])
-        assert_array_equal(y,[[1,2,3],[2,3,4],[3,4,5]])
 
 class TestPinv(TestCase):
 
@@ -425,5 +542,10 @@ class TestPinv(TestCase):
         a_pinv2 = pinv2(a)
         assert_array_almost_equal(a_pinv,a_pinv2)
 
+class TestNorm(object):
+    def test_zero_norm(self):
+        assert_equal(norm([1,0,3], 0), 2)
+        assert_equal(norm([1,2,3], 0), 3)
+
 if __name__ == "__main__":
     run_module_suite()
