@@ -14,10 +14,12 @@ Run tests if linalg is not installed:
   python tests/test_decomp.py
 """
 
-from numpy.testing import *
+import numpy as np
+from numpy.testing import TestCase, assert_equal, assert_array_almost_equal, \
+        assert_array_equal, assert_raises, run_module_suite, dec
 
-from scipy.linalg import eig,eigvals,lu,svd,svdvals,cholesky,qr, \
-     schur,rsf2csf, lu_solve,lu_factor,solve,diagsvd,hessenberg,rq, \
+from scipy.linalg import eig, eigvals, lu, svd, svdvals, cholesky, qr, \
+     schur, rsf2csf, lu_solve, lu_factor, solve, diagsvd, hessenberg, rq, \
      eig_banded, eigvals_banded, eigh
 from scipy.linalg.flapack import dgbtrf, dgbtrs, zgbtrf, zgbtrs, \
      dsbev, dsbevd, dsbevx, zhbevd, zhbevx
@@ -204,6 +206,18 @@ class TestEig(TestCase):
             if all(isfinite(res[:, i])):
                 assert_array_almost_equal(res[:, i], 0)
 
+    def test_not_square_error(self):
+        """Check that passing a non-square array raises a ValueError."""
+        A = np.arange(6).reshape(3,2)
+        assert_raises(ValueError, eig, A)
+
+    def test_shape_mismatch(self):
+        """Check that passing arrays of with different shapes raises a ValueError."""
+        A = identity(2)
+        B = np.arange(9.0).reshape(3,3)
+        assert_raises(ValueError, eig, A, B)
+        assert_raises(ValueError, eig, B, A)
+
 class TestEigBanded(TestCase):
 
     def __init__(self, *args):
@@ -777,54 +791,6 @@ class TestDiagSVD(TestCase):
     def test_simple(self):
         assert_array_almost_equal(diagsvd([1,0,0],3,3),[[1,0,0],[0,0,0],[0,0,0]])
 
-class TestCholesky(TestCase):
-
-    def test_simple(self):
-        a = [[8,2,3],[2,9,3],[3,3,6]]
-        c = cholesky(a)
-        assert_array_almost_equal(dot(transpose(c),c),a)
-        c = transpose(c)
-        a = dot(c,transpose(c))
-        assert_array_almost_equal(cholesky(a,lower=1),c)
-
-    def test_simple_complex(self):
-        m = array([[3+1j,3+4j,5],[0,2+2j,2+7j],[0,0,7+4j]])
-        a = dot(transpose(conjugate(m)),m)
-        c = cholesky(a)
-        a1 = dot(transpose(conjugate(c)),c)
-        assert_array_almost_equal(a,a1)
-        c = transpose(c)
-        a = dot(c,transpose(conjugate(c)))
-        assert_array_almost_equal(cholesky(a,lower=1),c)
-
-    def test_random(self):
-        n = 20
-        for k in range(2):
-            m = random([n,n])
-            for i in range(n):
-                m[i,i] = 20*(.1+m[i,i])
-            a = dot(transpose(m),m)
-            c = cholesky(a)
-            a1 = dot(transpose(c),c)
-            assert_array_almost_equal(a,a1)
-            c = transpose(c)
-            a = dot(c,transpose(c))
-            assert_array_almost_equal(cholesky(a,lower=1),c)
-
-    def test_random_complex(self):
-        n = 20
-        for k in range(2):
-            m = random([n,n])+1j*random([n,n])
-            for i in range(n):
-                m[i,i] = 20*(.1+abs(m[i,i]))
-            a = dot(transpose(conjugate(m)),m)
-            c = cholesky(a)
-            a1 = dot(transpose(conjugate(c)),c)
-            assert_array_almost_equal(a,a1)
-            c = transpose(c)
-            a = dot(c,transpose(conjugate(c)))
-            assert_array_almost_equal(cholesky(a,lower=1),c)
-
 
 class TestQR(TestCase):
 
@@ -1053,5 +1019,93 @@ class TestDataNotShared(TestCase):
         assert_equal(_datanotshared(A,M2),True)
 
 
+def test_aligned_mem_float():
+    """Check linalg works with non-aligned memory"""
+    # Allocate 402 bytes of memory (allocated on boundary)
+    a = arange(402, dtype=np.uint8)
+
+    # Create an array with boundary offset 4
+    z = np.frombuffer(a.data, offset=2, count=100, dtype=float32)
+    z.shape = 10, 10
+
+    eig(z, overwrite_a=True)
+    eig(z.T, overwrite_a=True)
+
+
+def test_aligned_mem():
+    """Check linalg works with non-aligned memory"""
+    # Allocate 804 bytes of memory (allocated on boundary)
+    a = arange(804, dtype=np.uint8)
+
+    # Create an array with boundary offset 4
+    z = np.frombuffer(a.data, offset=4, count=100, dtype=float)
+    z.shape = 10, 10
+
+    eig(z, overwrite_a=True)
+    eig(z.T, overwrite_a=True)
+
+def test_aligned_mem_complex():
+    """Check that complex objects don't need to be completely aligned"""
+    # Allocate 1608 bytes of memory (allocated on boundary)
+    a = zeros(1608, dtype=np.uint8)
+
+    # Create an array with boundary offset 8
+    z = np.frombuffer(a.data, offset=8, count=100, dtype=complex)
+    z.shape = 10, 10
+
+    eig(z, overwrite_a=True)
+    # This does not need special handling
+    eig(z.T, overwrite_a=True)
+
+def check_lapack_misaligned(func, args, kwargs):
+    args = list(args)
+    for i in range(len(args)):
+        a = args[:]
+        if isinstance(a[i],np.ndarray):
+            # Try misaligning a[i]
+            aa = np.zeros(a[i].size*a[i].dtype.itemsize+8, dtype=np.uint8)
+            aa = np.frombuffer(aa.data, offset=4, count=a[i].size, dtype=a[i].dtype)
+            aa.shape = a[i].shape
+            aa[...] = a[i]
+            a[i] = aa
+            func(*a,**kwargs)
+            if len(a[i].shape)>1:
+                a[i] = a[i].T
+                func(*a,**kwargs)
+
+
+@dec.knownfailureif(True, "Ticket #1152, triggers a segfault in rare cases.")
+def test_lapack_misaligned():
+    M = np.eye(10,dtype=float)
+    R = np.arange(100)
+    R.shape = 10,10
+    S = np.arange(20000,dtype=np.uint8)
+    S = np.frombuffer(S.data, offset=4, count=100, dtype=np.float)
+    S.shape = 10, 10
+    b = np.ones(10)
+    v = np.ones(3,dtype=float)
+    LU, piv = lu_factor(S)
+    for (func, args, kwargs) in [
+            (eig,(S,),dict(overwrite_a=True)), # crash
+            (eigvals,(S,),dict(overwrite_a=True)), # no crash
+            (lu,(S,),dict(overwrite_a=True)), # no crash
+            (lu_factor,(S,),dict(overwrite_a=True)), # no crash
+            (lu_solve,((LU,piv),b),dict(overwrite_b=True)),
+            (solve,(S,b),dict(overwrite_a=True,overwrite_b=True)),
+            (svd,(M,),dict(overwrite_a=True)), # no crash
+            (svd,(R,),dict(overwrite_a=True)), # no crash
+            (svd,(S,),dict(overwrite_a=True)), # crash
+            (svdvals,(S,),dict()), # no crash
+            (svdvals,(S,),dict(overwrite_a=True)), #crash
+            (cholesky,(M,),dict(overwrite_a=True)), # no crash
+            (qr,(S,),dict(overwrite_a=True)), # crash
+            (rq,(S,),dict(overwrite_a=True)), # crash
+            (hessenberg,(S,),dict(overwrite_a=True)), # crash
+            (schur,(S,),dict(overwrite_a=True)), # crash
+            ]:
+        yield check_lapack_misaligned, func, args, kwargs
+# not properly tested
+# cholesky, rsf2csf, lu_solve, solve, eig_banded, eigvals_banded, eigh, diagsvd
+
 if __name__ == "__main__":
     run_module_suite()
