@@ -58,7 +58,7 @@ def _get_output_fourier_complex(output, input):
         if input.dtype.type in [numpy.complex64, numpy.complex128]:
             output = numpy.zeros(input.shape, dtype = input.dtype)
         else:
-            output = numpy.zeros(input.shape, dtype = numpy.Complex64)
+            output = numpy.zeros(input.shape, dtype = numpy.complex128)
         return_value = output
     elif type(output) is types.TypeType:
         if output not in [numpy.complex64, numpy.complex128]:
@@ -72,15 +72,38 @@ def _get_output_fourier_complex(output, input):
     return output, return_value
 
 def fourier_gaussian(input, sigma, n = -1, axis = -1, output = None):
-    """Multi-dimensional Gaussian fourier filter.
+    """
+    Multi-dimensional Gaussian fourier filter.
 
     The array is multiplied with the fourier transform of a Gaussian
-    kernel. If the parameter n is negative, then the input is assumed to be
-    the result of a complex fft. If n is larger or equal to zero, the input
-    is assumed to be the result of a real fft, and n gives the length of
-    the of the array before transformation along the the real transform
-    direction. The axis of the real transform is given by the axis
-    parameter.
+    kernel.
+
+    Parameters
+    ----------
+    input : array_like
+        The input array.
+    sigma : float or sequence
+        The sigma of the Gaussian kernel. If a float, `sigma` is the same for
+        all axes. If a sequence, `sigma` has to contain one value for each
+        axis.
+    n : int, optional
+        If `n` is negative (default), then the input is assumed to be the
+        result of a complex fft.
+        If `n` is larger than or equal to zero, the input is assumed to be the
+        result of a real fft, and `n` gives the length of the array before
+        transformation along the real transform direction.
+    axis : int, optional
+        The axis of the real transform.
+    output : ndarray, optional
+        If given, the result of filtering the input is placed in this array.
+        None is returned in this case.
+
+    Returns
+    -------
+    return_value : ndarray or None
+        The filtered input. If `output` is given as a parameter, None is
+        returned.
+
     """
     input = numpy.asarray(input)
     output, return_value = _get_output_fourier(output, input)
@@ -94,15 +117,38 @@ def fourier_gaussian(input, sigma, n = -1, axis = -1, output = None):
     return return_value
 
 def fourier_uniform(input, size, n = -1, axis = -1, output = None):
-    """Multi-dimensional Uniform fourier filter.
+    """
+    Multi-dimensional uniform fourier filter.
 
     The array is multiplied with the fourier transform of a box of given
-    sizes. If the parameter n is negative, then the input is assumed to be
-    the result of a complex fft. If n is larger or equal to zero, the input
-    is assumed to be the result of a real fft, and n gives the length of
-    the of the array before transformation along the the real transform
-    direction. The axis of the real transform is given by the axis
-    parameter.
+    size.
+
+    Parameters
+    ----------
+    input : array_like
+        The input array.
+    size : float or sequence
+        The size of the box used for filtering.
+        If a float, `size` is the same for all axes. If a sequence, `size` has
+        to contain one value for each axis.
+    n : int, optional
+        If `n` is negative (default), then the input is assumed to be the
+        result of a complex fft.
+        If `n` is larger than or equal to zero, the input is assumed to be the
+        result of a real fft, and `n` gives the length of the array before
+        transformation along the real transform direction.
+    axis : int, optional
+        The axis of the real transform.
+    output : ndarray, optional
+        If given, the result of filtering the input is placed in this array.
+        None is returned in this case.
+
+    Returns
+    -------
+    return_value : ndarray or None
+        The filtered input. If `output` is given as a parameter, None is
+        returned.
+
     """
     input = numpy.asarray(input)
     output, return_value = _get_output_fourier(output, input)
@@ -115,16 +161,42 @@ def fourier_uniform(input, size, n = -1, axis = -1, output = None):
     return return_value
 
 def fourier_ellipsoid(input, size, n = -1, axis = -1, output = None):
-    """Multi-dimensional ellipsoid fourier filter.
+    """
+    Multi-dimensional ellipsoid fourier filter.
 
     The array is multiplied with the fourier transform of a ellipsoid of
-    given sizes. If the parameter n is negative, then the input is assumed
-    to be the result of a complex fft. If n is larger or equal to zero, the
-    input is assumed to be the result of a real fft, and n gives the length
-    of the of the array before transformation along the the real transform
-    direction. The axis of the real transform is given by the axis
-    parameter. This function is implemented for arrays of
-    rank 1, 2, or 3.
+    given sizes.
+
+    Parameters
+    ----------
+    input : array_like
+        The input array.
+    size : float or sequence
+        The size of the box used for filtering.
+        If a float, `size` is the same for all axes. If a sequence, `size` has
+        to contain one value for each axis.
+    n : int, optional
+        If `n` is negative (default), then the input is assumed to be the
+        result of a complex fft.
+        If `n` is larger than or equal to zero, the input is assumed to be the
+        result of a real fft, and `n` gives the length of the array before
+        transformation along the real transform direction.
+    axis : int, optional
+        The axis of the real transform.
+    output : ndarray, optional
+        If given, the result of filtering the input is placed in this array.
+        None is returned in this case.
+
+    Returns
+    -------
+    return_value : ndarray or None
+        The filtered input. If `output` is given as a parameter, None is
+        returned.
+
+    Notes
+    -----
+    This function is implemented for arrays of rank 1, 2, or 3.
+
     """
     input = numpy.asarray(input)
     output, return_value = _get_output_fourier(output, input)
@@ -137,16 +209,38 @@ def fourier_ellipsoid(input, size, n = -1, axis = -1, output = None):
     return return_value
 
 def fourier_shift(input, shift, n = -1, axis = -1, output = None):
-    """Multi-dimensional fourier shift filter.
+    """
+    Multi-dimensional fourier shift filter.
 
-    The array is multiplied with the fourier transform of a shift operation
-    If the parameter n is negative, then the input is assumed to be the
-    result of a complex fft. If n is larger or equal to zero, the input is
-    assumed to be the result of a real fft, and n gives the length of the
-    of the array before transformation along the the real transform
-    direction. The axis of the real transform is given by the axis
-    parameter.
-     """
+    The array is multiplied with the fourier transform of a shift operation.
+
+    Parameters
+    ----------
+    input : array_like
+        The input array.
+    shift : float or sequence
+        The size of the box used for filtering.
+        If a float, `shift` is the same for all axes. If a sequence, `shift`
+        has to contain one value for each axis.
+    n : int, optional
+        If `n` is negative (default), then the input is assumed to be the
+        result of a complex fft.
+        If `n` is larger than or equal to zero, the input is assumed to be the
+        result of a real fft, and `n` gives the length of the array before
+        transformation along the real transform direction.
+    axis : int, optional
+        The axis of the real transform.
+    output : ndarray, optional
+        If given, the result of shifting the input is placed in this array.
+        None is returned in this case.
+
+    Returns
+    -------
+    return_value : ndarray or None
+        The shifted input. If `output` is given as a parameter, None is
+        returned.
+
+    """
     input = numpy.asarray(input)
     output, return_value = _get_output_fourier_complex(output, input)
     axis = _ni_support._check_axis(axis, input.ndim)
