@@ -1,8 +1,6 @@
-## Automatically adapted for scipy Oct 07, 2005 by convertcode.py
 
 from scipy.optimize import minpack2
 import numpy
-import sys
 
 import __builtin__
 pymin = __builtin__.min
