@@ -4,6 +4,9 @@ Copyright 2005 Travis Oliphant
 Permission to use, copy, modify, and distribute this software without fee
 is granted under the SciPy License.
 */
+#include <Python.h>
+#define PY_ARRAY_UNIQUE_SYMBOL _scipy_signal_ARRAY_API
+#include <numpy/noprefix.h>
 
 #include "sigtools.h"
 #include <setjmp.h>
@@ -115,382 +118,6 @@ static int increment(intp *ret_ind, int nd, intp *max_ind) {
     return incr;
 }
 
-/*
- All of these MultAdd functions loop over all the elements of the smallest
- array, incrementing an array of indices into the large N-D array at
- the same time.  The
- bounds for the other array are checked and if valid the product is
- added to the running sum.  If invalid bounds are found nothing is
- done (zero is added).  This has the effect of zero padding the array
- to handle edges.
- */
-
-#define MAKE_MultAdd(NTYPE, ctype) \
-static void NTYPE ## _MultAdd(char *ip1, intp is1, char *ip2, intp is2, char *op, \
-			  intp *dims1, intp *dims2, int ndims, intp nels2, int check, \
-			  intp *loop_ind, intp *temp_ind, uintp *offset) \
-{									\
-  ctype tmp=(ctype)0.0;  intp i;					\
-  int k, incr = 1;							\
-  ctype *ptr1 = (ctype *)ip1, *ptr2 = (ctype *)ip2;			\
-\
-  i = nels2;					\
-\
-  temp_ind[ndims-1]--; \
-  while (i--) {  \
-    /* Adjust index array and move ptr1 to right place */ \
-    k = ndims - 1; \
-    while(--incr) { \
-      temp_ind[k] -= dims2[k] - 1;   /* Return to start for these dimensions */ \
-      k--; \
-    } \
-    ptr1 += offset[k];               /* Precomputed offset array */ \
-    temp_ind[k]++; \
-\
-    if (!(check && index_out_of_bounds(temp_ind,dims1,ndims))) {  \
-      tmp += (*ptr1) * (*ptr2);  \
-    } \
-    incr = increment(loop_ind, ndims, dims2);  /* Returns number of N-D indices incremented. */ \
-    ptr2++; \
-    \
-  }						\
-*((ctype *)op) = tmp;				\
-}
-
-MAKE_MultAdd(UBYTE, ubyte)
-MAKE_MultAdd(BYTE, byte)
-MAKE_MultAdd(USHORT, ushort)
-MAKE_MultAdd(SHORT, short)
-MAKE_MultAdd(UINT, uint)
-MAKE_MultAdd(INT, int)
-MAKE_MultAdd(ULONG, ulong)
-MAKE_MultAdd(LONG, long)
-MAKE_MultAdd(ULONGLONG, ulonglong)
-MAKE_MultAdd(LONGLONG, longlong)
-MAKE_MultAdd(FLOAT, float)
-MAKE_MultAdd(DOUBLE, double)
-MAKE_MultAdd(LONGDOUBLE, longdouble)
-
-#define MAKE_CMultAdd(NTYPE, ctype) \
-static void NTYPE ## _MultAdd(char *ip1, intp is1, char *ip2, intp is2, char *op, \
-			      intp *dims1, intp *dims2, int ndims, intp nels2, int check, \
-			      intp *loop_ind, intp *temp_ind, uintp *offset) \
-{ \
-  ctype tmpr= 0.0, tmpi = 0.0; intp i; \
-  int k, incr = 1;				    \
-  ctype *ptr1 = (ctype *)ip1, *ptr2 = (ctype *)ip2; \
- \
-  i = nels2; \
-\
-  temp_ind[ndims-1]--; \
-  while (i--) {  \
-    /* Adjust index array and move ptr1 to right place */ \
-    k = ndims - 1; \
-    while(--incr) { \
-      temp_ind[k] -= dims2[k] - 1;   /* Return to start for these dimensions */ \
-      k--; \
-    } \
-    ptr1 += 2*offset[k];               /* Precomputed offset array */ \
-    temp_ind[k]++; \
-\
-    if (!(check && index_out_of_bounds(temp_ind,dims1,ndims))) {  \
-      tmpr += ptr1[0] * ptr2[0] - ptr1[1] * ptr2[1];  \
-      tmpi += ptr1[1] * ptr2[0] + ptr1[0] * ptr2[1];  \
-    }  \
-    incr = increment(loop_ind, ndims, dims2);  \
-    /* Returns number of N-D indices incremented. */	\
-    ptr2 += 2; \
-\
-  } \
-  ((ctype *)op)[0] = tmpr; ((ctype *)op)[1] = tmpi; \
-} 
-
-MAKE_CMultAdd(CFLOAT, float)
-MAKE_CMultAdd(CDOUBLE, double)
-MAKE_CMultAdd(CLONGDOUBLE, longdouble)
-
-static void correlateND(Generic_Array *ap1, Generic_Array *ap2, Generic_Array *ret, MultAddFunction *multiply_and_add_ND, int mode) {
-  intp *a_ind, *b_ind, *temp_ind, *check_ind, *mode_dep;
-  uintp *offsets, offset1;
-  intp *offsets2;
-  int i, k, check, incr = 1;
-  int bytes_in_array, num_els_ret, num_els_ap2;
-  intp is1, is2, os;
-  char *ip1, *ip2, *op, *ap1_ptr;
-  intp *ret_ind;
-
-  num_els_ret = 1;
-  for (i = 0; i < ret->nd; i++) num_els_ret *= ret->dimensions[i];
-  num_els_ap2 = 1;
-  for (i = 0; i < ret->nd; i++) num_els_ap2 *= ap2->dimensions[i];
-  bytes_in_array = ap1->nd * sizeof(intp);
-  mode_dep = (intp *)malloc(bytes_in_array);
-  switch(mode) {
-  case 0:
-    for (i = 0; i < ap1->nd; i++) mode_dep[i] = 0;
-    break;
-  case 1:
-    for (i = 0; i < ap1->nd; i++) mode_dep[i] = -((ap2->dimensions[i]) >> 1);
-    break;
-  case 2:
-    for (i = 0; i < ap1->nd; i++) mode_dep[i] = 1 - ap2->dimensions[i];
-  }
-  
-  is1 = ap1->elsize; is2 = ap2->elsize;
-  op = ret->data; os = ret->elsize;
-  ip1 = ap1->data; ip2 = ap2->data;
-  op = ret->data;
-  
-  b_ind = (intp *)malloc(bytes_in_array);  /* loop variables */
-  memset(b_ind,0,bytes_in_array);
-  a_ind = (intp *)malloc(bytes_in_array);
-  ret_ind = (intp *)malloc(bytes_in_array);
-  memset(ret_ind,0,bytes_in_array);
-  temp_ind = (intp *)malloc(bytes_in_array);
-  check_ind = (intp *)malloc(bytes_in_array);
-  offsets = (uintp *)malloc(ap1->nd*sizeof(uintp));
-  offsets2 = (intp *)malloc(ap1->nd*sizeof(intp));
-  offset1 = compute_offsets(offsets,offsets2,ap1->dimensions,ap2->dimensions,ret->dimensions,mode_dep,ap1->nd);
-  /* The convolution proceeds by looping through the output array
-     and for each value summing all contributions from the summed
-     element-by-element product of the two input arrays.  Index
-     counters are used for book-keeping in the area so that we 
-     can tell where we are in all of the arrays and be sure that 
-     we are not trying to access areas outside the arrays definition.
-     
-     The inner loop is implemented separately but equivalently for each
-     datatype. The outer loop is similar in structure and form to
-     to the inner loop.
-  */
-  /* Need to keep track of a ptr to place in big (first) input
-     array where we start the multiplication (we pass over it in the
-     inner loop (and not dereferenced) 
-     if it is pointing outside dataspace)
-  */
-  /* Calculate it once and the just move it around appropriately */
-  ap1_ptr = ip1 + offset1*is1;
-  for (k=0; k < ap1->nd; k++) {a_ind[k] = mode_dep[k]; check_ind[k] = ap1->dimensions[k] - ap2->dimensions[k] - mode_dep[k] - 1;}
-  a_ind[ap1->nd-1]--;
-  i = num_els_ret;
-  while (i--) {
-    k = ap1->nd - 1;
-    while(--incr) {
-      a_ind[k] -= ret->dimensions[k] - 1;   /* Return to start */
-      k--;
-    }
-    ap1_ptr += offsets2[k]*is1;
-    a_ind[k]++;
-    memcpy(temp_ind, a_ind, bytes_in_array);
-    
-    check = 0; k = -1;
-    while(!check && (++k < ap1->nd))
-      check = check || (ret_ind[k] < -mode_dep[k]) || (ret_ind[k] > check_ind[k]);
-    
-    multiply_and_add_ND(ap1_ptr,is1,ip2,is2,op,ap1->dimensions,ap2->dimensions,ap1->nd,num_els_ap2,check,b_ind,temp_ind,offsets);
-    
-    incr = increment(ret_ind,ret->nd,ret->dimensions); /* increment index counter */
-    op += os;   /* increment to next output index */
-    
-  }
-  free(b_ind); free(a_ind); free(ret_ind);
-  free(offsets); free(offsets2); free(temp_ind);
-  free(check_ind); free(mode_dep);
-}
-
-/*****************************************************************
- *   This is code for a 1-D linear-filter along an arbitrary     *
- *   dimension of an N-D array.                                  *
- *****************************************************************/
-
-static void FLOAT_filt(char *b, char *a, char *x, char *y, char *Z, intp len_b, uintp len_x, intp stride_X, intp stride_Y ) {
-  char *ptr_x = x, *ptr_y = y;
-  float *ptr_Z, *ptr_b;
-  float *ptr_a;
-  float *xn, *yn;
-  const float a0 = *((float *)a);
-  int k, n;
-
-  for (k = 0; k < len_x; k++) {
-    ptr_b = (float *)b;          /* Reset a and b pointers */
-    ptr_a = (float *)a;
-    xn = (float *)ptr_x;
-    yn = (float *)ptr_y;
-    if (len_b > 1) {
-      ptr_Z = ((float *)Z);
-      *yn = *ptr_Z + *ptr_b / a0 * *xn;    /* Calculate first delay (output) */
-      ptr_b++; ptr_a++;
-    /* Fill in middle delays */
-      for (n = 0; n < len_b - 2; n++) {
-	*ptr_Z = ptr_Z[1] + *xn * (*ptr_b / a0) - *yn * (*ptr_a / a0);
-	ptr_b++; ptr_a++; ptr_Z++;
-      }
-    /* Calculate last delay */
-      *ptr_Z = *xn * (*ptr_b / a0) - *yn * (*ptr_a / a0);
-    }
-    else {
-      *yn = *xn * (*ptr_b / a0);
-    }
-
-    ptr_y += stride_Y;             /* Move to next input/output point */
-    ptr_x += stride_X;     
-  }
-}
-
-
-static void DOUBLE_filt(char *b, char *a, char *x, char *y, char *Z, intp len_b, uintp len_x, intp stride_X, intp stride_Y ) {
-  char *ptr_x = x, *ptr_y = y;
-  double *ptr_Z, *ptr_b;
-  double *ptr_a;
-  double *xn, *yn;
-  double a0;
-  int k, n;
-
-  a0 = *((double *)a);
-  for (k = 0; k < len_x; k++) {
-    ptr_b = (double *)b;          /* Reset a and b pointers */
-    ptr_a = (double *)a;
-    xn = (double *)ptr_x;
-    yn = (double *)ptr_y;
-    if (len_b > 1) {
-      ptr_Z = ((double *)Z);
-      *yn = *ptr_Z + *ptr_b / a0 * *xn;    /* Calculate first delay (output) */
-      ptr_b++; ptr_a++;
-      /* Fill in middle delays */
-      for (n = 0; n < len_b - 2; n++) {
-	*ptr_Z = ptr_Z[1] + *xn * (*ptr_b / a0) - *yn * (*ptr_a / a0);
-	ptr_b++; ptr_a++; ptr_Z++;
-      }
-    /* Calculate last delay */
-      *ptr_Z = *xn * (*ptr_b / a0) - *yn * (*ptr_a / a0);
-    }
-    else {
-      *yn = *xn * (*ptr_b / a0);
-    }
-
-    ptr_y += stride_Y;             /* Move to next input/output point */
-    ptr_x += stride_X;     
-  }
-}
-
- 
-static void CFLOAT_filt(char *b, char *a, char *x, char *y, char *Z, intp len_b, uintp len_x, intp stride_X, intp stride_Y ) {
-  char *ptr_x = x, *ptr_y = y;
-  float *ptr_Z, *ptr_b;
-  float *ptr_a;
-  float *xn, *yn;
-  float a0r = ((float *)a)[0];
-  float a0i = ((float *)a)[1];
-  float a0_mag, tmpr, tmpi;
-  int k, n;
-
-  a0_mag = a0r*a0r + a0i*a0i;
-  for (k = 0; k < len_x; k++) {
-    ptr_b = (float *)b;          /* Reset a and b pointers */
-    ptr_a = (float *)a;
-    xn = (float *)ptr_x;
-    yn = (float *)ptr_y;
-    if (len_b > 1) {
-      ptr_Z = ((float *)Z);
-      tmpr = ptr_b[0]*a0r + ptr_b[1]*a0i;
-      tmpi = ptr_b[1]*a0r - ptr_b[0]*a0i;
-      /* Calculate first delay (output) */
-      yn[0] = ptr_Z[0] + (tmpr * xn[0] - tmpi * xn[1])/a0_mag;    
-      yn[1] = ptr_Z[1] + (tmpi * xn[0] + tmpr * xn[1])/a0_mag;
-      ptr_b += 2; ptr_a += 2;
-      /* Fill in middle delays */
-      for (n = 0; n < len_b - 2; n++) {
-	tmpr = ptr_b[0]*a0r + ptr_b[1]*a0i;
-	tmpi = ptr_b[1]*a0r - ptr_b[0]*a0i;
-	ptr_Z[0] = ptr_Z[2] + (tmpr * xn[0] - tmpi * xn[1])/a0_mag;
-	ptr_Z[1] = ptr_Z[3] + (tmpi * xn[0] + tmpr * xn[1])/a0_mag;
-	tmpr = ptr_a[0]*a0r + ptr_a[1]*a0i;
-	tmpi = ptr_a[1]*a0r - ptr_a[0]*a0i;
-	ptr_Z[0] -= (tmpr * yn[0] - tmpi * yn[1])/a0_mag;
-	ptr_Z[1] -= (tmpi * yn[0] + tmpr * yn[1])/a0_mag;
-	ptr_b += 2; ptr_a += 2; ptr_Z += 2;
-      }
-      /* Calculate last delay */
-
-      tmpr = ptr_b[0]*a0r + ptr_b[1]*a0i;
-      tmpi = ptr_b[1]*a0r - ptr_b[0]*a0i;
-      ptr_Z[0] = (tmpr * xn[0] - tmpi * xn[1])/a0_mag;
-      ptr_Z[1] = (tmpi * xn[0] + tmpr * xn[1])/a0_mag;
-      tmpr = ptr_a[0]*a0r + ptr_a[1]*a0i;
-      tmpi = ptr_a[1]*a0r - ptr_a[0]*a0i;
-      ptr_Z[0] -= (tmpr * yn[0] - tmpi * yn[1])/a0_mag;
-      ptr_Z[1] -= (tmpi * yn[0] + tmpr * yn[1])/a0_mag;
-    }
-    else {
-      tmpr = ptr_b[0]*a0r + ptr_b[1]*a0i;
-      tmpi = ptr_b[1]*a0r - ptr_b[0]*a0i;
-      yn[0] = (tmpr * xn[0] - tmpi * xn[1])/a0_mag;
-      yn[1] = (tmpi * xn[0] + tmpr * xn[1])/a0_mag;      
-    }
-
-    ptr_y += stride_Y;             /* Move to next input/output point */
-    ptr_x += stride_X;     
-
-  }
-}
-
-
-static void CDOUBLE_filt(char *b, char *a, char *x, char *y, char *Z, intp len_b, uintp len_x, intp stride_X, intp stride_Y ) {
-  char *ptr_x = x, *ptr_y = y;
-  double *ptr_Z, *ptr_b;
-  double *ptr_a;
-  double *xn, *yn;
-  double a0r = ((double *)a)[0];
-  double a0i = ((double *)a)[1];
-  double a0_mag, tmpr, tmpi;
-  int k, n;
-
-  a0_mag = a0r*a0r + a0i*a0i;
-  for (k = 0; k < len_x; k++) {
-    ptr_b = (double *)b;          /* Reset a and b pointers */
-    ptr_a = (double *)a;
-    xn = (double *)ptr_x;
-    yn = (double *)ptr_y;
-    if (len_b > 1) {
-      ptr_Z = ((double *)Z);
-      tmpr = ptr_b[0]*a0r + ptr_b[1]*a0i;
-      tmpi = ptr_b[1]*a0r - ptr_b[0]*a0i;
-      /* Calculate first delay (output) */
-      yn[0] = ptr_Z[0] + (tmpr * xn[0] - tmpi * xn[1])/a0_mag;    
-      yn[1] = ptr_Z[1] + (tmpi * xn[0] + tmpr * xn[1])/a0_mag;
-      ptr_b += 2; ptr_a += 2;
-      /* Fill in middle delays */
-      for (n = 0; n < len_b - 2; n++) {
-	tmpr = ptr_b[0]*a0r + ptr_b[1]*a0i;
-	tmpi = ptr_b[1]*a0r - ptr_b[0]*a0i;
-	ptr_Z[0] = ptr_Z[2] + (tmpr * xn[0] - tmpi * xn[1])/a0_mag;
-	ptr_Z[1] = ptr_Z[3] + (tmpi * xn[0] + tmpr * xn[1])/a0_mag;
-	tmpr = ptr_a[0]*a0r + ptr_a[1]*a0i;
-	tmpi = ptr_a[1]*a0r - ptr_a[0]*a0i;
-	ptr_Z[0] -= (tmpr * yn[0] - tmpi * yn[1])/a0_mag;
-	ptr_Z[1] -= (tmpi * yn[0] + tmpr * yn[1])/a0_mag;
-	ptr_b += 2; ptr_a += 2; ptr_Z += 2;
-      }
-      /* Calculate last delay */
-      tmpr = ptr_b[0]*a0r + ptr_b[1]*a0i;
-      tmpi = ptr_b[1]*a0r - ptr_b[0]*a0i;
-      ptr_Z[0] = (tmpr * xn[0] - tmpi * xn[1])/a0_mag;
-      ptr_Z[1] = (tmpi * xn[0] + tmpr * xn[1])/a0_mag;
-      tmpr = ptr_a[0]*a0r + ptr_a[1]*a0i;
-      tmpi = ptr_a[1]*a0r - ptr_a[0]*a0i;
-      ptr_Z[0] -= (tmpr * yn[0] - tmpi * yn[1])/a0_mag;
-      ptr_Z[1] -= (tmpi * yn[0] + tmpr * yn[1])/a0_mag;
-    }
-    else {
-      tmpr = ptr_b[0]*a0r + ptr_b[1]*a0i;
-      tmpi = ptr_b[1]*a0r - ptr_b[0]*a0i;
-      yn[0] = (tmpr * xn[0] - tmpi * xn[1])/a0_mag;
-      yn[1] = (tmpi * xn[0] + tmpr * xn[1])/a0_mag;      
-    } 
-    ptr_y += stride_Y;             /* Move to next input/output point */
-    ptr_x += stride_X;     
-  }
-}
-
 /********************************************************
  *
  *  Code taken from remez.c by Erik Kvaleberg which was 
@@ -1135,108 +762,6 @@ static int pre_remez(double *h2, int numtaps, int numbands, double *bands, doubl
 /* End of python-independent routines               */
 /****************************************************/
 
-static void OBJECT_MultAdd(char *ip1, intp is1, char *ip2, intp is2, char *op, intp *dims1, intp *dims2, int ndims, intp nels2, int check, intp *loop_ind, intp *temp_ind, uintp *offset) { 
-  int i, k, first_time = 1, incr = 1; 
-  PyObject *tmp1=NULL, *tmp2=NULL, *tmp=NULL;
-
-  i = nels2;
-
-  temp_ind[ndims-1]--;
-  while (i--) { 
-    /* Adjust index array and move ptr1 to right place */
-    k = ndims - 1;
-    while(--incr) {
-      temp_ind[k] -= dims2[k] - 1;   /* Return to start for these dimensions */
-      k--;
-    }
-    ip1 += offset[k]*is1;           /* Precomputed offset array */
-    temp_ind[k]++;
-
-    if (!(check && index_out_of_bounds(temp_ind,dims1,ndims))) { 
-      tmp1 = PyNumber_Multiply(*((PyObject **)ip1),*((PyObject **)ip2));
-      if (first_time) {
-	tmp = tmp1;
-	first_time = 0;
-      } else {
-	tmp2 = PyNumber_Add(tmp, tmp1);
-	Py_XDECREF(tmp);
-	tmp = tmp2;
-	Py_XDECREF(tmp1);
-      }
-    } 
-    incr = increment(loop_ind, ndims, dims2); 
-    ip2 += is2; 
-  }
-  Py_XDECREF(*((PyObject **)op));
-  *((PyObject **)op) = tmp; 
-}
-
-static MultAddFunction *MultiplyAddFunctions[] = 
-  {NULL, BYTE_MultAdd, UBYTE_MultAdd, SHORT_MultAdd,
-   USHORT_MultAdd,INT_MultAdd, UINT_MultAdd, LONG_MultAdd,
-   ULONG_MultAdd, LONGLONG_MultAdd, ULONGLONG_MultAdd,
-   FLOAT_MultAdd, DOUBLE_MultAdd, LONGDOUBLE_MultAdd,
-   CFLOAT_MultAdd, CDOUBLE_MultAdd, CLONGDOUBLE_MultAdd,
-   OBJECT_MultAdd, NULL, NULL, NULL};
-
-static void OBJECT_filt(char *b, char *a, char *x, char *y, char *Z, intp len_b, uintp len_x, intp stride_X, intp stride_Y ) {
-  char *ptr_x = x, *ptr_y = y;
-  PyObject  **ptr_Z, **ptr_b;
-  PyObject  **ptr_a;
-  PyObject  **xn, **yn;
-  PyObject  **a0 = (PyObject  **)a;
-  PyObject  *tmp1, *tmp2, *tmp3;
-  int k, n;
-
-  /* My reference counting might not be right */
-  for (k = 0; k < len_x; k++) {
-    ptr_b = (PyObject  **)b;          /* Reset a and b pointers */
-    ptr_a = (PyObject  **)a;
-    xn = (PyObject  **)ptr_x;
-    yn = (PyObject  **)ptr_y;
-    if (len_b > 1) {
-      ptr_Z = ((PyObject  **)Z);
-      /* Calculate first delay (output) */
-      tmp1 = PyNumber_Multiply(*ptr_b,*xn);
-      tmp2 = PyNumber_Divide(tmp1,*a0);
-      tmp3 = PyNumber_Add(tmp2,*ptr_Z);
-      Py_XDECREF(*yn);
-      *yn = tmp3; Py_DECREF(tmp1); Py_DECREF(tmp2);
-      ptr_b++; ptr_a++;
-
-      /* Fill in middle delays */
-      for (n = 0; n < len_b - 2; n++) {
-	tmp1 = PyNumber_Multiply(*xn, *ptr_b);
-	tmp2 = PyNumber_Divide(tmp1,*a0);
-	tmp3 = PyNumber_Add(tmp2,ptr_Z[1]);
-	Py_DECREF(tmp1); Py_DECREF(tmp2);
-	tmp1 = PyNumber_Multiply(*yn, *ptr_a);
-	tmp2 = PyNumber_Divide(tmp1, *a0); Py_DECREF(tmp1);
-	Py_XDECREF(*ptr_Z);
-	*ptr_Z = PyNumber_Subtract(tmp3, tmp2); Py_DECREF(tmp2);
-	Py_DECREF(tmp3);
-	ptr_b++; ptr_a++; ptr_Z++;
-      }
-    /* Calculate last delay */
-      tmp1 = PyNumber_Multiply(*xn,*ptr_b);
-      tmp3 = PyNumber_Divide(tmp1,*a0); Py_DECREF(tmp1);
-      tmp1 = PyNumber_Multiply(*yn, *ptr_a);
-      tmp2 = PyNumber_Divide(tmp1, *a0); Py_DECREF(tmp1);
-      Py_XDECREF(*ptr_Z);
-      *ptr_Z = PyNumber_Subtract(tmp3,tmp2); Py_DECREF(tmp2);
-      Py_DECREF(tmp3);
-    }
-    else {
-      tmp1 = PyNumber_Multiply(*xn,*ptr_b);
-      Py_XDECREF(*yn);
-      *yn = PyNumber_Divide(tmp1,*a0); Py_DECREF(tmp1);
-    }
-
-    ptr_y += stride_Y;             /* Move to next input/output point */
-    ptr_x += stride_X;     
-  }
-}
-
 /************************/
 /* N-D Order Filtering. */
 
@@ -1466,135 +991,17 @@ fail:
 }
 
 
-static BasicFilterFunction *BasicFilterFunctions[] = \
-	{NULL, NULL,NULL,NULL,NULL,NULL,NULL, NULL, NULL, NULL, NULL, \
-	 FLOAT_filt, DOUBLE_filt, NULL,				      \
-	 CFLOAT_filt, CDOUBLE_filt, NULL, \
-	 OBJECT_filt, NULL, NULL, NULL}; 
-/* There is the start of an OBJECT_filt, but it may need work */
-
-
-/* Copy data from PyArray to Generic header for use in C routines */
-static void Py_copy_info(Generic_Array *gen, PyArrayObject *py_arr) {
-        gen->data = py_arr->data;
-	gen->nd = py_arr->nd;
-	gen->dimensions = py_arr->dimensions;
-	gen->elsize = py_arr->descr->elsize;
-	gen->strides = py_arr->strides;
-	gen->zero = PyArray_Zero(py_arr);
-	return;
-}
-
-static void Py_copy_info_vec(Generic_Vector *gen, PyArrayObject *py_arr) {
-        gen->data = py_arr->data;
-	gen->elsize = py_arr->descr->elsize;
-	gen->numels = PyArray_Size((PyObject *)py_arr);
-	gen->zero = PyArray_Zero(py_arr);
-	return;
-}
-
 /******************************************/
 
 static char doc_correlateND[] = "out = _correlateND(a,kernel,mode) \n\n   mode = 0 - 'valid', 1 - 'same', \n  2 - 'full' (default)";
 
-static PyObject *sigtools_correlateND(PyObject *dummy, PyObject *args) {
-	PyObject *kernel, *a0;
-	PyArrayObject *ap1, *ap2, *ret;
-	Generic_Array in1, in2, out;
-	intp *ret_dimens;
-	int mode=2, n1, n2, i, typenum;
-	MultAddFunction *multiply_and_add_ND;
-	
-	if (!PyArg_ParseTuple(args, "OO|i", &a0, &kernel, &mode)) return NULL;
-
-	typenum = PyArray_ObjectType(a0, 0);  
-	typenum = PyArray_ObjectType(kernel, typenum);
-	
-	ret = NULL;
-	ap1 = (PyArrayObject *)PyArray_ContiguousFromObject(a0, typenum, 0, 0);
-	if (ap1 == NULL) return NULL;
-	ap2 = (PyArrayObject *)PyArray_ContiguousFromObject(kernel, typenum, 0, 0);
-	if (ap2 == NULL) goto fail;
-
-	if (ap1->nd != ap2->nd) {
-	  PyErr_SetString(PyExc_ValueError, "Arrays must have the same number of dimensions.");
-	  goto fail;
-	}
-
-        if (ap1->nd == 0) {  /* Zero-dimensional arrays */
-          PyErr_SetString(PyExc_ValueError, "Cannot convolve zero-dimensional arrays.");
-          goto fail;
-        }
-	
-	n1 = PyArray_Size((PyObject *)ap1);
-	n2 = PyArray_Size((PyObject *)ap2);
-
-	/* Swap if first argument is not the largest */
-	if (n1 < n2) { ret = ap1; ap1 = ap2; ap2 = ret; ret = NULL; }
-	ret_dimens = malloc(ap1->nd*sizeof(intp));
-	switch(mode) {
-	case 0:
-	        for (i = 0; i < ap1->nd; i++) { 
-		  ret_dimens[i] = ap1->dimensions[i] - ap2->dimensions[i] + 1;
-		  if (ret_dimens[i] < 0) {
-		    PyErr_SetString(PyExc_ValueError, "no part of the output is valid, use option 1 (same) or 2 (full) for third argument");
-		    goto fail;
-		  }
-		}
-		break;
-	case 1:
-	        for (i = 0; i < ap1->nd; i++) { ret_dimens[i] = ap1->dimensions[i];}
-		break;
-	case 2:
-	        for (i = 0; i < ap1->nd; i++) { ret_dimens[i] = ap1->dimensions[i] + ap2->dimensions[i] - 1;}
-		break;
-	default: 
-	        PyErr_SetString(PyExc_ValueError, 
-			    "mode must be 0 (valid), 1 (same), or 2 (full)");
-		goto fail;
-	}
-	
-	ret = (PyArrayObject *)PyArray_SimpleNew(ap1->nd, ret_dimens, typenum);
-	free(ret_dimens);
-	if (ret == NULL) goto fail;
-	
-	multiply_and_add_ND = MultiplyAddFunctions[(int)(ret->descr->type_num)];
-	if (multiply_and_add_ND == NULL) {
-		PyErr_SetString(PyExc_ValueError, 
-			"correlateND not available for this type");
-		goto fail;
-	}
-
-	/* copy header information to generic structures */
-	Py_copy_info(&in1, ap1);
-	Py_copy_info(&in2, ap2);
-	Py_copy_info(&out, ret);
-       
-	correlateND(&in1, &in2, &out, multiply_and_add_ND, mode);
-
-	PyDataMem_FREE(in1.zero);
-	PyDataMem_FREE(in2.zero);
-	PyDataMem_FREE(out.zero);
-
-	Py_DECREF(ap1);
-	Py_DECREF(ap2);
-	return PyArray_Return(ret);
-
-fail:
-	Py_XDECREF(ap1);
-	Py_XDECREF(ap2);
-	Py_XDECREF(ret);
-	return NULL;	
-}
-
-
 /*******************************************************************/
 
 static char doc_convolve2d[] = "out = _convolve2d(in1, in2, flip, mode, boundary, fillvalue)";
 
 extern int pylab_convolve_2d(char*,intp*,char*,intp*,char*,intp*,intp*,intp*,int,char*);
 
-static PyObject *sigtools_convolve2d(PyObject *dummy, PyObject *args) {
+static PyObject *sigtools_convolve2d(PyObject *NPY_UNUSED(dummy), PyObject *args) {
 
     PyObject *in1=NULL, *in2=NULL, *fill_value=NULL;
     int mode=2, boundary=0, typenum, flag, flip=1, ret;
@@ -1719,7 +1126,7 @@ fail:
 
 static char doc_order_filterND[] = "out = _order_filterND(a,domain,order)";
 
-static PyObject *sigtools_order_filterND(PyObject *dummy, PyObject *args) {
+static PyObject *sigtools_order_filterND(PyObject *NPY_UNUSED(dummy), PyObject *args) {
 	PyObject *domain, *a0;
 	int order=0;
 	
@@ -1732,7 +1139,7 @@ static PyObject *sigtools_order_filterND(PyObject *dummy, PyObject *args) {
 
 static char doc_remez[] = "h = _remez(numtaps, bands, des, weight, type, Hz, maxiter, grid_density) \n  returns the optimal (in the Chebyshev/minimax sense) FIR filter impulse \n  response given a set of band edges, the desired response on those bands,\n  and the weight given to the error in those bands.  Bands is a monotonic\n   vector with band edges given in frequency domain where Hz is the sampling\n   frequency.";
  
-static PyObject *sigtools_remez(PyObject *dummy, PyObject *args) {
+static PyObject *sigtools_remez(PyObject *NPY_UNUSED(dummy), PyObject *args) {
         PyObject *bands, *des, *weight;
         int k, numtaps, numbands, type = BANDPASS, err; 
 	PyArrayObject *a_bands=NULL, *a_des=NULL, *a_weight=NULL;
@@ -1832,7 +1239,7 @@ extern void f_medfilt2(float*,float*,intp*,intp*);
 extern void d_medfilt2(double*,double*,intp*,intp*);
 extern void b_medfilt2(unsigned char*,unsigned char*,intp*,intp*);
 
-static PyObject *sigtools_median2d(PyObject *dummy, PyObject *args)
+static PyObject *sigtools_median2d(PyObject *NPY_UNUSED(dummy), PyObject *args)
 {
     PyObject *image=NULL, *size=NULL;
     int typenum;
@@ -1847,7 +1254,7 @@ static PyObject *sigtools_median2d(PyObject *dummy, PyObject *args)
     if (a_image == NULL) goto fail;
 
     if (size != NULL) {
-	a_size = (PyArrayObject *)PyArray_ContiguousFromObject(size, PyArray_LONG, 1, 1);
+	a_size = (PyArrayObject *)PyArray_ContiguousFromObject(size, NPY_INTP, 1, 1);
 	if (a_size == NULL) goto fail;
 	if ((RANK(a_size) != 1) || (DIMS(a_size)[0] < 2)) 
 	    PYERR("Size must be a length two sequence");
@@ -1890,16 +1297,19 @@ static PyObject *sigtools_median2d(PyObject *dummy, PyObject *args)
 
 }
 
-#include "lfilter.c"
+static char doc_linear_filter[] =
+    "(y,Vf) = _linear_filter(b,a,X,Dim=-1,Vi=None)  " \
+    "implemented using Direct Form II transposed flow " \
+    "diagram. If Vi is not given, Vf is not returned.";
 
 static struct PyMethodDef toolbox_module_methods[] = {
-	{"_correlateND", sigtools_correlateND, METH_VARARGS, doc_correlateND},
+	{"_correlateND", scipy_signal_sigtools_correlateND, METH_VARARGS, doc_correlateND},
 	{"_convolve2d", sigtools_convolve2d, METH_VARARGS, doc_convolve2d},
 	{"_order_filterND", sigtools_order_filterND, METH_VARARGS, doc_order_filterND},
-	{"_linear_filter",sigtools_linear_filter, METH_VARARGS, doc_linear_filter},
+	{"_linear_filter", scipy_signal_sigtools_linear_filter, METH_VARARGS, doc_linear_filter},
 	{"_remez",sigtools_remez, METH_VARARGS, doc_remez},
 	{"_medfilt2d", sigtools_median2d, METH_VARARGS, doc_median2d},
-	{NULL,		NULL, 0}		/* sentinel */
+	{NULL, NULL, 0, NULL}		/* sentinel */
 };
 
 /* Initialization function for the module (*must* be called initsigtools) */
@@ -1928,6 +1338,7 @@ PyMODINIT_FUNC initsigtools(void) {
         PyDict_SetItemString(d,"HILBERT", PyInt_FromLong((long) HILBERT));
         */
 
+        scipy_signal_sigtools_linear_filter_module_init();
 
 	/* Check for errors */
 	if (PyErr_Occurred()) {
