@@ -13,12 +13,13 @@ from base import spmatrix, isspmatrix
 from sputils import isdense, getdtype, isshape, isintlike, isscalarlike, upcast
 
 class dok_matrix(spmatrix, dict):
-    """Dictionary Of Keys based sparse matrix.
+    """
+    Dictionary Of Keys based sparse matrix.
 
     This is an efficient structure for constructing sparse
     matrices incrementally.
 
-    This can be instatiated in several ways:
+    This can be instantiated in several ways:
         dok_matrix(D)
             with a dense matrix, D
 
@@ -218,7 +219,7 @@ class dok_matrix(spmatrix, dict):
                 raise IndexError, "index out of bounds"
 
             if np.isscalar(value):
-                if value==0:
+                if value==0 and self.has_key((i,j)):
                     del self[(i,j)]
                 else:
                     dict.__setitem__(self, (i,j), self.dtype.type(value))
