@@ -550,8 +550,18 @@ class _TestCommon:
         assert_equal(toself(copy=False).todense(), A.todense())
 
 
-        #TODO how can we check whether the data is copied?
-        pass
+        # check whether the data is copied?
+        # TODO: deal with non-indexable types somehow
+        B = A.copy()
+        try:
+            B[0,0] += 1
+            assert B[0,0]!=A[0,0]
+        except NotImplementedError:
+            # not all sparse matrices can be indexed
+            pass
+        except TypeError:
+            # not all sparse matrices can be indexed
+            pass
 
     # Eventually we'd like to allow matrix products between dense
     # and sparse matrices using the normal dot() function:
@@ -592,6 +602,7 @@ class _TestInplaceArithmetic:
 class _TestGetSet:
     def test_setelement(self):
         A = self.spmatrix((3,4))
+        A[ 0, 0] = 0 # bug 870
         A[ 1, 2] = 4.0
         A[ 0, 1] = 3
         A[ 2, 0] = 2.0
@@ -738,6 +749,23 @@ class _TestFancyIndexing:
     """Tests fancy indexing features.  The tests for any matrix formats
     that implement these features should derive from this class.
     """
+    def test_fancy_indexing_set(self):
+        n, m = (5, 10)
+        def _test_set(i, j, nitems):
+            A = self.spmatrix((n, m))
+            A[i, j] = 1
+            assert_almost_equal(A.sum(), nitems)
+            assert_almost_equal(A[i, j], 1)
+
+        # [i,j]
+        for i, j in [(2, 3), (-1, 8), (-1, -2), (array(-1), -2), (-1, array(-2)), 
+                     (array(-1), array(-2))]:
+            _test_set(i, j, 1)
+
+        # [i,1:2]
+        for i, j in [(2, slice(m)), (2, slice(5, -2)), (array(2), slice(5, -2))]:
+            _test_set(i, j, 3)
+
     def test_fancy_indexing(self):
         B = asmatrix(arange(50).reshape(5,10))
         A = self.spmatrix( B )
@@ -837,6 +865,30 @@ class _TestFancyIndexing:
         s = slice(int8(2),int8(4),None)
         assert_equal(A[s,:].todense(), B[2:4,:])
         assert_equal(A[:,s].todense(), B[:,2:4])
+    
+    def test_fancy_indexing_randomized(self):
+        random.seed(0) # make runs repeatable
+
+        NUM_SAMPLES = 50
+        M = 6
+        N = 4
+
+        D = np.asmatrix(np.random.rand(M,N))
+        D = np.multiply(D, D > 0.5)
+
+        I = np.random.random_integers(-M + 1, M - 1, size=NUM_SAMPLES)
+        J = np.random.random_integers(-N + 1, N - 1, size=NUM_SAMPLES)
+        
+        S = self.spmatrix(D)
+        
+        assert_equal(S[I,J], D[I,J])
+
+        I_bad = I + M
+        J_bad = J - N
+
+        assert_raises(IndexError, S.__getitem__, (I_bad,J))
+        assert_raises(IndexError, S.__getitem__, (I,J_bad))
+
 
 class _TestArithmetic:
     """
@@ -925,6 +977,11 @@ class TestCSR(_TestCommon, _TestGetSet, _TestSolve,
         _TestFancyIndexing, TestCase):
     spmatrix = csr_matrix
 
+    @dec.knownfailureif(True, "Fancy indexing is known to be broken for CSR" \
+                              " matrices")
+    def test_fancy_indexing_set(self):
+        _TestFancyIndexing.test_fancy_indexing_set(self)
+
     def test_constructor1(self):
         b = matrix([[0,4,0],
                    [3,0,0],
@@ -992,7 +1049,6 @@ class TestCSR(_TestCommon, _TestGetSet, _TestSolve,
         csr = csr_matrix((data, indices, indptr))
         assert_array_equal(csr.shape,(3,6))
 
-
     def test_sort_indices(self):
         data    = arange( 5 )
         indices = array( [7, 2, 1, 5, 4] )
@@ -1014,6 +1070,18 @@ class TestCSR(_TestCommon, _TestGetSet, _TestSolve,
         assert_array_equal(asp.data,[1, 2, 3])
         assert_array_equal(asp.todense(),bsp.todense())
 
+    def test_unsorted_arithmetic(self):
+        data    = arange( 5 )
+        indices = array( [7, 2, 1, 5, 4] )
+        indptr  = array( [0, 3, 5] )
+        asp = csr_matrix( (data, indices, indptr), shape=(2,10) )
+        data    = arange( 6 )
+        indices = array( [8, 1, 5, 7, 2, 4] )
+        indptr  = array( [0, 2, 6] )
+        bsp = csr_matrix( (data, indices, indptr), shape=(2,10) )
+        assert_equal((asp + bsp).todense(), asp.todense() + bsp.todense())
+
+
 
 
 class TestCSC(_TestCommon, _TestGetSet, _TestSolve,
@@ -1022,6 +1090,11 @@ class TestCSC(_TestCommon, _TestGetSet, _TestSolve,
         _TestFancyIndexing, TestCase):
     spmatrix = csc_matrix
 
+    @dec.knownfailureif(True, "Fancy indexing is known to be broken for CSC" \
+                              " matrices")
+    def test_fancy_indexing_set(self):
+        _TestFancyIndexing.test_fancy_indexing_set(self)
+
     def test_constructor1(self):
         b = matrix([[1,0,0,0],[0,0,1,0],[0,2,0,3]],'d')
         bsp = csc_matrix(b)
@@ -1087,6 +1160,16 @@ class TestCSC(_TestCommon, _TestGetSet, _TestSolve,
         assert_array_equal(asp.indices,[1, 2, 7, 4, 5])
         assert_array_equal(asp.todense(),bsp.todense())
 
+    def test_unsorted_arithmetic(self):
+        data    = arange( 5 )
+        indices = array( [7, 2, 1, 5, 4] )
+        indptr  = array( [0, 3, 5] )
+        asp = csc_matrix( (data, indices, indptr), shape=(10,2) )
+        data    = arange( 6 )
+        indices = array( [8, 1, 5, 7, 2, 4] )
+        indptr  = array( [0, 2, 6] )
+        bsp = csc_matrix( (data, indices, indptr), shape=(10,2) )
+        assert_equal((asp + bsp).todense(), asp.todense() + bsp.todense())
 
 class TestDOK(_TestCommon, _TestGetSet, _TestSolve, TestCase):
     spmatrix = dok_matrix
@@ -1216,7 +1299,7 @@ class TestDOK(_TestCommon, _TestGetSet, _TestSolve, TestCase):
 
 class TestLIL( _TestCommon, _TestHorizSlicing, _TestVertSlicing,
         _TestBothSlicing, _TestGetSet, _TestSolve,
-        _TestArithmetic, _TestInplaceArithmetic,
+        _TestArithmetic, _TestInplaceArithmetic, _TestFancyIndexing,
         TestCase):
     spmatrix = lil_matrix
 
@@ -1226,6 +1309,17 @@ class TestLIL( _TestCommon, _TestHorizSlicing, _TestVertSlicing,
     B[2,1] = 3
     B[3,0] = 10
 
+    
+    @dec.knownfailureif(True, "Fancy indexing is known to be broken for LIL" \
+                              " matrices")
+    def test_fancy_indexing_set(self):
+        _TestFancyIndexing.test_fancy_indexing_set(self)
+
+    @dec.knownfailureif(True, "Fancy indexing is known to be broken for LIL" \
+                              " matrices")
+    def test_fancy_indexing_randomized(self):
+        _TestFancyIndexing.test_fancy_indexing_randomized(self)
+
     def test_dot(self):
         A = matrix(zeros((10,10)))
         A[0,3] = 10
@@ -1301,7 +1395,7 @@ class TestLIL( _TestCommon, _TestHorizSlicing, _TestVertSlicing,
         B[:2,:2] = csc_matrix(array(block))
         assert_array_equal(B.todense()[:2,:2],block)
 
-    def test_lil_sequence_assignement(self):
+    def test_lil_sequence_assignment(self):
         A = lil_matrix((4,3))
         B = eye(3,4,format='lil')
 
@@ -1314,6 +1408,16 @@ class TestLIL( _TestCommon, _TestHorizSlicing, _TestVertSlicing,
         A[2,i2] = B[i2,2]
         assert_array_equal(A.todense(),B.T.todense())
 
+        # column slice
+        A = lil_matrix((2,3))
+        A[1,1:3] = [10,20]
+        assert_array_equal(A.todense(), [[0,0,0],[0,10,20]])
+
+        # column slice
+        A = lil_matrix((3,2))
+        A[1:3,1] = [[10],[20]]
+        assert_array_equal(A.todense(), [[0,0],[0,10],[0,20]])
+
     def test_lil_iteration(self):
         row_data = [[1,2,3],[4,5,6]]
         B = lil_matrix(array(row_data))
