@@ -28,14 +28,7 @@ from scipy.special import *
 import scipy.special._cephes as cephes
 import numpy as np
 
-def assert_tol_equal(a, b, rtol=1e-7, atol=0, err_msg='', verbose=True):
-    """Assert that `a` and `b` are equal to tolerance ``atol + rtol*abs(b)``"""
-    def compare(x, y):
-        return allclose(x, y, rtol=rtol, atol=atol)
-    a, b = asanyarray(a), asanyarray(b)
-    header = 'Not equal to tolerance rtol=%g, atol=%g' % (rtol, atol)
-    np.testing.utils.assert_array_compare(compare, a, b, err_msg=str(err_msg),
-                                          verbose=verbose, header=header)
+from testutils import *
 
 class TestCephes(TestCase):
     def test_airy(self):
@@ -340,14 +333,16 @@ class TestCephes(TestCase):
         cephes.nctdtrit(.1,0.2,.5)
 
     def test_ndtr(self):
-        assert_equal(cephes.ndtr(0),0.5)
+        assert_equal(cephes.ndtr(0), 0.5)
+        assert_almost_equal(cephes.ndtr(1), 0.84134474606)
+
     def test_ndtri(self):
         assert_equal(cephes.ndtri(0.5),0.0)
     def test_nrdtrimn(self):
         assert_approx_equal(cephes.nrdtrimn(0.5,1,1),1.0)
     def test_nrdtrisd(self):
-        # abs() because nrdtrisd(0.5,0.5,0.5) returns -0.0, should be +0.0
-        assert_equal(np.abs(cephes.nrdtrisd(0.5,0.5,0.5)), 0.0)
+        assert_tol_equal(cephes.nrdtrisd(0.5,0.5,0.5), 0.0,
+                         atol=0, rtol=0)
 
     def test_obl_ang1(self):
         cephes.obl_ang1(1,1,1,0)
@@ -417,6 +412,15 @@ class TestCephes(TestCase):
         cephes.shichi(1)
     def test_sici(self):
         cephes.sici(1)
+
+        s, c = cephes.sici(np.inf)
+        assert_almost_equal(s, np.pi * 0.5)
+        assert_almost_equal(c, 0)
+
+        s, c = cephes.sici(-np.inf)
+        assert_almost_equal(s, -np.pi * 0.5)
+        assert_(np.isnan(c), "cosine integral(-inf) is not nan")
+
     def test_sindg(self):
         assert_equal(cephes.sindg(90),1.0)
     def test_smirnov(self):
@@ -709,64 +713,6 @@ class TestBeta(TestCase):
         comp = betainc(2,4,y)
         assert_almost_equal(comp,.5,5)
 
-class TestCheby(TestCase):
-    def test_chebyc(self):
-        C0 = chebyc(0)
-        C1 = chebyc(1)
-        C2 = chebyc(2)
-        C3 = chebyc(3)
-        C4 = chebyc(4)
-        C5 = chebyc(5)
-
-        assert_array_almost_equal(C0.c,[2],13)
-        assert_array_almost_equal(C1.c,[1,0],13)
-        assert_array_almost_equal(C2.c,[1,0,-2],13)
-        assert_array_almost_equal(C3.c,[1,0,-3,0],13)
-        assert_array_almost_equal(C4.c,[1,0,-4,0,2],13)
-        assert_array_almost_equal(C5.c,[1,0,-5,0,5,0],13)
-
-    def test_chebys(self):
-        S0 = chebys(0)
-        S1 = chebys(1)
-        S2 = chebys(2)
-        S3 = chebys(3)
-        S4 = chebys(4)
-        S5 = chebys(5)
-        assert_array_almost_equal(S0.c,[1],13)
-        assert_array_almost_equal(S1.c,[1,0],13)
-        assert_array_almost_equal(S2.c,[1,0,-1],13)
-        assert_array_almost_equal(S3.c,[1,0,-2,0],13)
-        assert_array_almost_equal(S4.c,[1,0,-3,0,1],13)
-        assert_array_almost_equal(S5.c,[1,0,-4,0,3,0],13)
-
-    def test_chebyt(self):
-        T0 = chebyt(0)
-        T1 = chebyt(1)
-        T2 = chebyt(2)
-        T3 = chebyt(3)
-        T4 = chebyt(4)
-        T5 = chebyt(5)
-        assert_array_almost_equal(T0.c,[1],13)
-        assert_array_almost_equal(T1.c,[1,0],13)
-        assert_array_almost_equal(T2.c,[2,0,-1],13)
-        assert_array_almost_equal(T3.c,[4,0,-3,0],13)
-        assert_array_almost_equal(T4.c,[8,0,-8,0,1],13)
-        assert_array_almost_equal(T5.c,[16,0,-20,0,5,0],13)
-
-    def test_chebyu(self):
-        U0 = chebyu(0)
-        U1 = chebyu(1)
-        U2 = chebyu(2)
-        U3 = chebyu(3)
-        U4 = chebyu(4)
-        U5 = chebyu(5)
-        assert_array_almost_equal(U0.c,[1],13)
-        assert_array_almost_equal(U1.c,[2,0],13)
-        assert_array_almost_equal(U2.c,[4,0,-1],13)
-        assert_array_almost_equal(U3.c,[8,0,-4,0],13)
-        assert_array_almost_equal(U4.c,[16,0,-12,0,1],13)
-        assert_array_almost_equal(U5.c,[32,0,-32,0,6,0],13)
-
 class TestTrigonometric(TestCase):
     def test_cbrt(self):
         cb = cbrt(27)
@@ -870,7 +816,7 @@ class TestTandg(TestCase):
 
 class TestEllip(TestCase):
     def test_ellipj_nan(self):
-        """Regression test for #946."""
+        """Regression test for #912."""
         ellipj(0.5, np.nan)
 
     def test_ellipj(self):
@@ -1054,6 +1000,7 @@ class TestGamma(TestCase):
         gcinv = gammaincinv(.5,.5)
         assert_almost_equal(gccinv,gcinv,8)
 
+    @with_special_errors
     def test_gammaincinv(self):
         y = gammaincinv(.4,.4)
         x = gammainc(.4,y)
@@ -1065,6 +1012,19 @@ class TestGamma(TestCase):
         x = gammaincinv(50, 8.20754777388471303050299243573393e-18)
         assert_almost_equal(11.0, x, decimal=10)
 
+    @with_special_errors
+    def test_975(self):
+        # Regression test for ticket #975 -- switch point in algorithm
+        # check that things work OK at the point, immediately next floats
+        # around it, and a bit further away
+        pts = [0.25,
+               np.nextafter(0.25, 0), 0.25 - 1e-12,
+               np.nextafter(0.25, 1), 0.25 + 1e-12]
+        for xp in pts:
+            y = gammaincinv(.4, xp)
+            x = gammainc(0.4, y)
+            assert_tol_equal(x, xp, rtol=1e-12)
+
     def test_rgamma(self):
         rgam = rgamma(8)
         rlgam = 1/gamma(8)
@@ -1103,69 +1063,6 @@ class TestHankel(TestCase):
         hankrl2e = hankel2e(1,.1)
         assert_almost_equal(hank2e,hankrl2e,8)
 
-class TestHermite(TestCase):
-    def test_hermite(self):
-        H0 = hermite(0)
-        H1 = hermite(1)
-        H2 = hermite(2)
-        H3 = hermite(3)
-        H4 = hermite(4)
-        H5 = hermite(5)
-        assert_array_almost_equal(H0.c,[1],13)
-        assert_array_almost_equal(H1.c,[2,0],13)
-        assert_array_almost_equal(H2.c,[4,0,-2],13)
-        assert_array_almost_equal(H3.c,[8,0,-12,0],13)
-        assert_array_almost_equal(H4.c,[16,0,-48,0,12],12)
-        assert_array_almost_equal(H5.c,[32,0,-160,0,120,0],12)
-
-    def test_hermitenorm(self):
-        # He_n(x) = 2**(-n/2) H_n(x/sqrt(2))
-        psub = poly1d([1.0/sqrt(2),0])
-        H0 = hermitenorm(0)
-        H1 = hermitenorm(1)
-        H2 = hermitenorm(2)
-        H3 = hermitenorm(3)
-        H4 = hermitenorm(4)
-        H5 = hermitenorm(5)
-        he0 = hermite(0)(psub)
-        he1 = hermite(1)(psub) / sqrt(2)
-        he2 = hermite(2)(psub) / 2.0
-        he3 = hermite(3)(psub) / (2*sqrt(2))
-        he4 = hermite(4)(psub) / 4.0
-        he5 = hermite(5)(psub) / (4.0*sqrt(2))
-
-        assert_array_almost_equal(H0.c,he0.c,13)
-        assert_array_almost_equal(H1.c,he1.c,13)
-        assert_array_almost_equal(H2.c,he2.c,13)
-        assert_array_almost_equal(H3.c,he3.c,13)
-        assert_array_almost_equal(H4.c,he4.c,13)
-        assert_array_almost_equal(H5.c,he5.c,13)
-
-_gam = cephes.gamma
-
-class TestGegenbauer(TestCase):
-
-    def test_gegenbauer(self):
-        a = 5*rand()-0.5
-        if any(a==0): a = -0.2
-        Ca0 = gegenbauer(0,a)
-        Ca1 = gegenbauer(1,a)
-        Ca2 = gegenbauer(2,a)
-        Ca3 = gegenbauer(3,a)
-        Ca4 = gegenbauer(4,a)
-        Ca5 = gegenbauer(5,a)
-
-        assert_array_almost_equal(Ca0.c,array([1]),13)
-        assert_array_almost_equal(Ca1.c,array([2*a,0]),13)
-        assert_array_almost_equal(Ca2.c,array([2*a*(a+1),0,-a]),13)
-        assert_array_almost_equal(Ca3.c,array([4*poch(a,3),0,-6*a*(a+1),
-                                               0])/3.0,11)
-        assert_array_almost_equal(Ca4.c,array([4*poch(a,4),0,-12*poch(a,3),
-                                               0,3*a*(a+1)])/6.0,11)
-        assert_array_almost_equal(Ca5.c,array([4*poch(a,5),0,-20*poch(a,4),
-                                               0,15*poch(a,3),0])/15.0,11)
-
-
 class TestHyper(TestCase):
     def test_h1vp(self):
         h1 = h1vp(1,.1)
@@ -1318,6 +1215,14 @@ class TestHyper(TestCase):
                   # and some others
                   # ticket #424
                   [1.5, -0.5, 1.0, -10.0, 4.1300097765277476484],
+                  # negative integer a or b, with c-a-b integer and x > 0.9
+                  [-2,3,1,0.95,0.715],
+                  [2,-3,1,0.95,-0.007],
+                  [-6,3,1,0.95,0.0000810625],
+                  [2,-5,1,0.95,-0.000029375],
+                  # huge negative integers
+                  (10, -900,  10.5, 0.99, 1.91853705796607664803709475658e-24),
+                  (10, -900, -10.5, 0.99, 3.54279200040355710199058559155e-18),
                   ]
         for i, (a, b, c, x, v) in enumerate(values):
             cv = hyp2f1(a, b, c, x)
@@ -1410,7 +1315,7 @@ class TestBessel(TestCase):
                                        123.70194191713507279,
                                        129.02417238949092824,
                                        134.00114761868422559]), rtol=1e-13)
-
+        
         jn301 = jn_zeros(301,5)
         assert_tol_equal(jn301, array([313.59097866698830153,
                                        323.21549776096288280,
@@ -1423,7 +1328,7 @@ class TestBessel(TestCase):
         assert_tol_equal(jn0[260-1], 816.02884495068867280, rtol=1e-13)
         assert_tol_equal(jn0[280-1], 878.86068707124422606, rtol=1e-13)
         assert_tol_equal(jn0[300-1], 941.69253065317954064, rtol=1e-13)
-
+        
         jn10 = jn_zeros(10, 300)
         assert_tol_equal(jn10[260-1], 831.67668514305631151, rtol=1e-13)
         assert_tol_equal(jn10[280-1], 894.51275095371316931, rtol=1e-13)
@@ -1598,7 +1503,7 @@ class TestBessel(TestCase):
         an = yn_zeros(4,2)
         assert_array_almost_equal(an,array([ 5.64515,  9.36162]),5)
         an = yn_zeros(443,5)
-        assert_tol_equal(an, [450.13573091578090314, 463.05692376675001542,
+        assert_tol_equal(an, [450.13573091578090314, 463.05692376675001542, 
                               472.80651546418663566, 481.27353184725625838,
                               488.98055964441374646], rtol=1e-15)
 
@@ -1652,7 +1557,7 @@ class TestBessel(TestCase):
             for z in [-1300, -11, -10, -1, 1., 10., 200.5, 401., 600.5,
                       700.6, 1300, 10003]:
                 yield v, z
-
+                
         # check half-integers; these are problematic points at least
         # for cephes/iv
         for v in 0.5 + arange(-60, 60):
@@ -1687,7 +1592,7 @@ class TestBessel(TestCase):
         self.check_cephes_vs_amos(yv, yn, rtol=1e-11, atol=1e-305, skip=skipper)
 
     def test_iv_cephes_vs_amos(self):
-        self.check_cephes_vs_amos(iv, iv, rtol=1e-12, atol=1e-305)
+        self.check_cephes_vs_amos(iv, iv, rtol=5e-9, atol=1e-305)
 
     @dec.slow
     def test_iv_cephes_vs_amos_mass_test(self):
@@ -1702,6 +1607,10 @@ class TestBessel(TestCase):
         c1 = iv(v, x)
         c2 = iv(v, x+0j)
 
+        # deal with differences in the inf cutoffs
+        c1[abs(c1) > 1e300] = np.inf
+        c2[abs(c2) > 1e300] = np.inf
+
         dc = abs(c1/c2 - 1)
         dc[np.isnan(dc)] = 0
 
@@ -1709,7 +1618,7 @@ class TestBessel(TestCase):
 
         # Most error apparently comes from AMOS and not our implementation;
         # there are some problems near integer orders there
-        assert dc[k] < 1e-9, (iv(v[k], x[k]), iv(v[k], x[k]+0j))
+        assert dc[k] < 1e-9, (v[k], x[k], iv(v[k], x[k]), iv(v[k], x[k]+0j))
 
     def test_kv_cephes_vs_amos(self):
         #self.check_cephes_vs_amos(kv, kn, rtol=1e-9, atol=1e-305)
@@ -1745,12 +1654,12 @@ class TestBessel(TestCase):
         assert_tol_equal(iv(-2,   1+0j), 0.1357476697670383)
         assert_tol_equal(kv(-1,   1+0j), 0.6019072301972347)
         assert_tol_equal(kv(-2,   1+0j), 1.624838898635178)
-
+        
         assert_tol_equal(jv(-0.5, 1+0j), 0.43109886801837607952)
         assert_tol_equal(jv(-0.5, 1+1j), 0.2628946385649065-0.827050182040562j)
         assert_tol_equal(yv(-0.5, 1+0j), 0.6713967071418031)
         assert_tol_equal(yv(-0.5, 1+1j), 0.967901282890131+0.0602046062142816j)
-
+        
         assert_tol_equal(iv(-0.5, 1+0j), 1.231200214592967)
         assert_tol_equal(iv(-0.5, 1+1j), 0.77070737376928+0.39891821043561j)
         assert_tol_equal(kv(-0.5, 1+0j), 0.4610685044478945)
@@ -1877,7 +1786,7 @@ class TestBessel(TestCase):
         y=(iv(0,2)+iv(2,2))/2
         x = ivp(1,2)
         assert_almost_equal(x,y,10)
-
+    
 
 class TestLaguerre(TestCase):
     def test_laguerre(self):
@@ -2049,7 +1958,7 @@ class TestParabolicCylinder(TestCase):
         eps = 1e-7 + 1e-7*abs(x)
         dp = (pbdv(eta, x + eps)[0] - pbdv(eta, x - eps)[0]) / eps / 2.
         assert_tol_equal(p[1], dp, rtol=1e-6, atol=1e-6)
-
+        
     def test_pbvv_gradient(self):
         x = np.linspace(-4, 4, 8)[:,None]
         eta = np.linspace(-10, 10, 5)[None,:]
@@ -2058,7 +1967,7 @@ class TestParabolicCylinder(TestCase):
         eps = 1e-7 + 1e-7*abs(x)
         dp = (pbvv(eta, x + eps)[0] - pbvv(eta, x - eps)[0]) / eps / 2.
         assert_tol_equal(p[1], dp, rtol=1e-6, atol=1e-6)
-
+        
 
 class TestPolygamma(TestCase):
     # from Table 6.2 (pg. 271) of A&S
@@ -2112,113 +2021,42 @@ class TestRound(TestCase):
         # correctly written.
         rndrl = (10,10,10,11)
         assert_array_equal(rnd,rndrl)
+        
 
-class _test_sh_legendre(TestCase):
+def test_sph_harm():
+    # Tests derived from tables in
+    # http://en.wikipedia.org/wiki/Table_of_spherical_harmonics
+    sh = sph_harm
+    pi = np.pi
+    exp = np.exp
+    sqrt = np.sqrt
+    sin = np.sin
+    cos = np.cos
+    yield (assert_array_almost_equal, sh(0,0,0,0),
+           0.5/sqrt(pi))
+    yield (assert_array_almost_equal, sh(-2,2,0.,pi/4),
+           0.25*sqrt(15./(2.*pi))*
+           (sin(pi/4))**2.)
+    yield (assert_array_almost_equal, sh(-2,2,0.,pi/2),
+           0.25*sqrt(15./(2.*pi)))
+    yield (assert_array_almost_equal, sh(2,2,pi,pi/2),
+           0.25*sqrt(15/(2.*pi))*
+           exp(0+2.*pi*1j)*sin(pi/2.)**2.)
+    yield (assert_array_almost_equal, sh(2,4,pi/4.,pi/3.),
+           (3./8.)*sqrt(5./(2.*pi))*
+           exp(0+2.*pi/4.*1j)*
+           sin(pi/3.)**2.*
+           (7.*cos(pi/3.)**2.-1))
+    yield (assert_array_almost_equal, sh(4,4,pi/8.,pi/6.),
+           (3./16.)*sqrt(35./(2.*pi))*
+           exp(0+4.*pi/8.*1j)*sin(pi/6.)**4.)
 
-    def test_sh_legendre(self):
-        # P*_n(x) = P_n(2x-1)
-        psub = poly1d([2,-1])
-        Ps0 = sh_legendre(0)
-        Ps1 = sh_legendre(1)
-        Ps2 = sh_legendre(2)
-        Ps3 = sh_legendre(3)
-        Ps4 = sh_legendre(4)
-        Ps5 = sh_legendre(5)
-        pse0 = legendre(0)(psub)
-        pse1 = legendre(1)(psub)
-        pse2 = legendre(2)(psub)
-        pse3 = legendre(3)(psub)
-        pse4 = legendre(4)(psub)
-        pse5 = legendre(5)(psub)
-        assert_array_almost_equal(Ps0.c,pse0.c,13)
-        assert_array_almost_equal(Ps1.c,pse1.c,13)
-        assert_array_almost_equal(Ps2.c,pse2.c,13)
-        assert_array_almost_equal(Ps3.c,pse3.c,13)
-        assert_array_almost_equal(Ps4.c,pse4.c,12)
-        assert_array_almost_equal(Ps5.c,pse5.c,12)
-
-class _test_sh_chebyt(TestCase):
-
-    def test_sh_chebyt(self):
-        # T*_n(x) = T_n(2x-1)
-        psub = poly1d([2,-1])
-        Ts0 = sh_chebyt(0)
-        Ts1 = sh_chebyt(1)
-        Ts2 = sh_chebyt(2)
-        Ts3 = sh_chebyt(3)
-        Ts4 = sh_chebyt(4)
-        Ts5 = sh_chebyt(5)
-        tse0 = chebyt(0)(psub)
-        tse1 = chebyt(1)(psub)
-        tse2 = chebyt(2)(psub)
-        tse3 = chebyt(3)(psub)
-        tse4 = chebyt(4)(psub)
-        tse5 = chebyt(5)(psub)
-        assert_array_almost_equal(Ts0.c,tse0.c,13)
-        assert_array_almost_equal(Ts1.c,tse1.c,13)
-        assert_array_almost_equal(Ts2.c,tse2.c,13)
-        assert_array_almost_equal(Ts3.c,tse3.c,13)
-        assert_array_almost_equal(Ts4.c,tse4.c,12)
-        assert_array_almost_equal(Ts5.c,tse5.c,12)
-
-
-class _test_sh_chebyu(TestCase):
-
-    def test_sh_chebyu(self):
-        # U*_n(x) = U_n(2x-1)
-        psub = poly1d([2,-1])
-        Us0 = sh_chebyu(0)
-        Us1 = sh_chebyu(1)
-        Us2 = sh_chebyu(2)
-        Us3 = sh_chebyu(3)
-        Us4 = sh_chebyu(4)
-        Us5 = sh_chebyu(5)
-        use0 = chebyu(0)(psub)
-        use1 = chebyu(1)(psub)
-        use2 = chebyu(2)(psub)
-        use3 = chebyu(3)(psub)
-        use4 = chebyu(4)(psub)
-        use5 = chebyu(5)(psub)
-        assert_array_almost_equal(Us0.c,use0.c,13)
-        assert_array_almost_equal(Us1.c,use1.c,13)
-        assert_array_almost_equal(Us2.c,use2.c,13)
-        assert_array_almost_equal(Us3.c,use3.c,13)
-        assert_array_almost_equal(Us4.c,use4.c,12)
-        assert_array_almost_equal(Us5.c,use5.c,11)
-
-class _test_sh_jacobi(TestCase):
-
-    def test_sh_jacobi(self):
-        # G^(p,q)_n(x) = n! gamma(n+p)/gamma(2*n+p) * P^(p-q,q-1)_n(2*x-1)
-        conv = lambda n,p: _gam(n+1)*_gam(n+p)/_gam(2*n+p)
-        psub = poly1d([2,-1])
-        q = 4*rand()
-        p = q-1 + 2*rand()
-        #print "shifted jacobi p,q = ", p, q
-        G0 = sh_jacobi(0,p,q)
-        G1 = sh_jacobi(1,p,q)
-        G2 = sh_jacobi(2,p,q)
-        G3 = sh_jacobi(3,p,q)
-        G4 = sh_jacobi(4,p,q)
-        G5 = sh_jacobi(5,p,q)
-        ge0 = jacobi(0,p-q,q-1)(psub) * conv(0,p)
-        ge1 = jacobi(1,p-q,q-1)(psub) * conv(1,p)
-        ge2 = jacobi(2,p-q,q-1)(psub) * conv(2,p)
-        ge3 = jacobi(3,p-q,q-1)(psub) * conv(3,p)
-        ge4 = jacobi(4,p-q,q-1)(psub) * conv(4,p)
-        ge5 = jacobi(5,p-q,q-1)(psub) * conv(5,p)
-
-        assert_array_almost_equal(G0.c,ge0.c,13)
-        assert_array_almost_equal(G1.c,ge1.c,13)
-        assert_array_almost_equal(G2.c,ge2.c,13)
-        assert_array_almost_equal(G3.c,ge3.c,13)
-        assert_array_almost_equal(G4.c,ge4.c,13)
-        assert_array_almost_equal(G5.c,ge5.c,13)
 
 class TestSpherical(TestCase):
     def test_sph_harm(self):
+        # see test_sph_harm function
         pass
-
+    
     def test_sph_in(self):
         i1n = sph_in(1,.2)
         inp0 = (i1n[0][1])
@@ -2300,5 +2138,15 @@ class TestStruve(object):
         assert_tol_equal(struve(-2.0, 20 - 1e-8), struve(-2.0, 20 + 1e-8))
         assert_tol_equal(struve(-4.3, 20 - 1e-8), struve(-4.3, 20 + 1e-8))
 
+def test_chi2_smalldf():
+    assert_almost_equal(chdtr(0.6,3), 0.957890536704110)
+
+def test_chi2c_smalldf():
+    assert_almost_equal(chdtrc(0.6,3), 1-0.957890536704110)
+
+def test_chi2_inv_smalldf():
+    assert_almost_equal(chdtri(0.6,1-0.957890536704110), 3)
+
+
 if __name__ == "__main__":
     run_module_suite()
