@@ -6,8 +6,10 @@ from numpy.testing import *
 
 
 import numpy
+import numpy as np
 from numpy import typecodes, array
 import scipy.stats as stats
+from scipy.stats.distributions import argsreduce
 
 def kolmogorov_check(diststr,args=(),N=20,significance=0.01):
     qtest = stats.ksoneisf(significance,N)
@@ -65,12 +67,30 @@ def test_all_distributions():
             vals[1] = vals[0] + 1.0
             args = tuple(vals)
         elif dist == 'vonmises':
-            yield check_distribution, dist, (100,), alpha
+            yield check_distribution, dist, (10,), alpha
+            yield check_distribution, dist, (101,), alpha
             args = tuple(1.0+rand(nargs))
         else:
             args = tuple(1.0+rand(nargs))
         yield check_distribution, dist, args, alpha
 
+def check_vonmises_pdf_periodic(k,l,s,x):
+    vm = stats.vonmises(k,loc=l,scale=s)
+    assert_almost_equal(vm.pdf(x),vm.pdf(x%(2*numpy.pi*s)))
+def check_vonmises_cdf_periodic(k,l,s,x):
+    vm = stats.vonmises(k,loc=l,scale=s)
+    assert_almost_equal(vm.cdf(x)%1,vm.cdf(x%(2*numpy.pi*s))%1)
+
+def test_vonmises_pdf_periodic():
+    for k in [0.1, 1, 101]:
+        for x in [0,1,numpy.pi,10,100]:
+            yield check_vonmises_pdf_periodic, k, 0, 1, x
+            yield check_vonmises_pdf_periodic, k, 1, 1, x
+            yield check_vonmises_pdf_periodic, k, 0, 10, x
+
+            yield check_vonmises_cdf_periodic, k, 0, 1, x
+            yield check_vonmises_cdf_periodic, k, 1, 1, x
+            yield check_vonmises_cdf_periodic, k, 0, 10, x
 
 class TestRandInt(TestCase):
     def test_rvs(self):
@@ -261,6 +281,72 @@ class TestExponpow(TestCase):
         assert_almost_equal(stats.exponpow.cdf(1e-10,  2.),  1e-20)
         assert_almost_equal(stats.exponpow.isf(stats.exponpow.sf(5, .8), .8),  5)
 
+
+class TestSkellam(TestCase):
+    def test_pmf(self):
+        #comparison to R
+        k = numpy.arange(-10, 15)
+        mu1, mu2 = 10, 5
+        skpmfR = numpy.array(
+                   [4.2254582961926893e-005, 1.1404838449648488e-004,
+                    2.8979625801752660e-004, 6.9177078182101231e-004,
+                    1.5480716105844708e-003, 3.2412274963433889e-003,
+                    6.3373707175123292e-003, 1.1552351566696643e-002,
+                    1.9606152375042644e-002, 3.0947164083410337e-002,
+                    4.5401737566767360e-002, 6.1894328166820688e-002,
+                    7.8424609500170578e-002, 9.2418812533573133e-002,
+                    1.0139793148019728e-001, 1.0371927988298846e-001,
+                    9.9076583077406091e-002, 8.8546660073089561e-002,
+                    7.4187842052486810e-002, 5.8392772862200251e-002,
+                    4.3268692953013159e-002, 3.0248159818374226e-002,
+                    1.9991434305603021e-002, 1.2516877303301180e-002,
+                    7.4389876226229707e-003])
+        
+        assert_almost_equal(stats.skellam.pmf(k, mu1, mu2), skpmfR, decimal=15)
+        
+    def test_cdf(self):
+        #comparison to R, only 5 decimals
+        k = numpy.arange(-10, 15)
+        mu1, mu2 = 10, 5
+        skcdfR = numpy.array(
+                   [6.4061475386192104e-005, 1.7810985988267694e-004,
+                    4.6790611790020336e-004, 1.1596768997212152e-003,
+                    2.7077485103056847e-003, 5.9489760066490718e-003,
+                    1.2286346724161398e-002, 2.3838698290858034e-002,
+                    4.3444850665900668e-002, 7.4392014749310995e-002,
+                    1.1979375231607835e-001, 1.8168808048289900e-001,
+                    2.6011268998306952e-001, 3.5253150251664261e-001,
+                    4.5392943399683988e-001, 5.5764871387982828e-001,
+                    6.5672529695723436e-001, 7.4527195703032389e-001,
+                    8.1945979908281064e-001, 8.7785257194501087e-001,
+                    9.2112126489802404e-001, 9.5136942471639818e-001,
+                    9.7136085902200120e-001, 9.8387773632530240e-001,
+                    9.9131672394792536e-001])
+
+        assert_almost_equal(stats.skellam.cdf(k, mu1, mu2), skcdfR, decimal=5)
+
+
+class TestHypergeom(TestCase):
+    def test_precision(self):
+        # comparison number from mpmath
+
+        M,n,N = 2500,50,500
+        tot=M;good=n;bad=tot-good
+        hgpmf = stats.hypergeom.pmf(2,tot,good,N)
+
+        assert_almost_equal(hgpmf, 0.0010114963068932233, 11)
+
+class TestChi2(TestCase):
+    # regression tests after precision improvements, ticket:1041, not verified
+    def test_precision(self):
+        assert_almost_equal(stats.chi2.pdf(1000, 1000), 8.919133934753128e-003, 14)
+        assert_almost_equal(stats.chi2.pdf(100, 100), 0.028162503162596778, 14)
+
+class TestArrayArgument(TestCase): #test for ticket:992
+    def test_noexception(self):
+        rvs = stats.norm.rvs(loc=(np.arange(5)), scale=np.ones(5), size=(10,5))
+        assert_equal(rvs.shape, (10,5))
+
 class TestDocstring(TestCase):
     def test_docstrings(self):
         """See ticket #761"""
@@ -279,5 +365,21 @@ class TestEntropy(TestCase):
         assert(0.0 == eself)
         assert(edouble >= 0.0)
 
+def TestArgsreduce():
+    a = array([1,3,2,1,2,3,3])
+    b,c = argsreduce(a > 1, a, 2)
+
+    assert_array_equal(b, [3,2,2,3,3])
+    assert_array_equal(c, [2,2,2,2,2])
+
+    b,c = argsreduce(2 > 1, a, 2)
+    assert_array_equal(b, a[0])
+    assert_array_equal(c, [2])
+
+    b,c = argsreduce(a > 0, a, 2)
+    assert_array_equal(b, a)
+    assert_array_equal(c, [2] * numpy.size(a))
+
+
 if __name__ == "__main__":
     run_module_suite()
