@@ -1,19 +1,18 @@
-import time
-import os,sys
+import os
+import sys
 
 # Note: test_dir is global to this file.
-#       It is made by setup_test_location()
+#       It is made by setup_location()
 
 #globals
 global test_dir
 test_dir = ''
 
 from numpy.testing import *
-set_package_path()
-from weave import inline_tools,ext_tools,c_spec
-from weave.build_tools import msvc_exists, gcc_exists
-from weave.catalog import unique_file
-restore_path()
+
+from scipy.weave import inline_tools,ext_tools,c_spec
+from scipy.weave.build_tools import msvc_exists, gcc_exists
+from scipy.weave.catalog import unique_file
 
 
 def unique_mod(d,file_name):
@@ -22,47 +21,52 @@ def unique_mod(d,file_name):
     return m
 
 def remove_whitespace(in_str):
-    import string
-    out = string.replace(in_str," ","")
-    out = string.replace(out,"\t","")
-    out = string.replace(out,"\n","")
+    out = in_str.replace(" ","")
+    out = out.replace("\t","")
+    out = out.replace("\n","")
     return out
 
-def print_assert_equal(test_string,actual,desired):
-    """this should probably be in scipy_test.testing
-    """
-    import pprint
-    try:
-        assert(actual == desired)
-    except AssertionError:
-        import cStringIO
-        msg = cStringIO.StringIO()
-        msg.write(test_string)
-        msg.write(' failed\nACTUAL: \n')
-        pprint.pprint(actual,msg)
-        msg.write('DESIRED: \n')
-        pprint.pprint(desired,msg)
-        raise AssertionError, msg.getvalue()
-
 #----------------------------------------------------------------------------
 # Scalar conversion test classes
 #   int, float, complex
 #----------------------------------------------------------------------------
-class test_int_converter(NumpyTestCase):
+# compilers = []
+# for c in ('gcc','msvc'):
+#     mod_name = 'empty' + c
+#     mod_name = unique_mod(test_dir,mod_name)
+#     mod = ext_tools.ext_module(mod_name)
+#     # a = 1
+#     # code = "a=2;"
+#     # test = ext_tools.ext_function('test',code,['a'])
+#     # mod.add_function(test)
+#     try:
+#         mod.compile(location = test_dir, compiler = c)
+#     except CompileError:
+#         print "Probably don't have Compiler: %s"%c
+#     else:
+#         compilers.append(c)
+
+
+class IntConverter(TestCase):
     compiler = ''
-    def check_type_match_string(self,level=5):
+    @dec.slow
+    def test_type_match_string(self):
         s = c_spec.int_converter()
         assert( not s.type_match('string') )
-    def check_type_match_int(self,level=5):
+    @dec.slow
+    def test_type_match_int(self):
         s = c_spec.int_converter()
         assert(s.type_match(5))
-    def check_type_match_float(self,level=5):
+    @dec.slow
+    def test_type_match_float(self):
         s = c_spec.int_converter()
         assert(not s.type_match(5.))
-    def check_type_match_complex(self,level=5):
+    @dec.slow
+    def test_type_match_complex(self):
         s = c_spec.int_converter()
         assert(not s.type_match(5.+1j))
-    def check_var_in(self,level=5):
+    @dec.slow
+    def test_var_in(self):
         mod_name = 'int_var_in' + self.compiler
         mod_name = unique_mod(test_dir,mod_name)
         mod = ext_tools.ext_module(mod_name)
@@ -85,7 +89,8 @@ class test_int_converter(NumpyTestCase):
         except TypeError:
             pass
 
-    def check_int_return(self,level=5):
+    @dec.slow
+    def test_int_return(self):
         mod_name = sys._getframe().f_code.co_name + self.compiler
         mod_name = unique_mod(test_dir,mod_name)
         mod = ext_tools.ext_module(mod_name)
@@ -103,21 +108,26 @@ class test_int_converter(NumpyTestCase):
 
         assert( c == 3)
 
-class test_float_converter(NumpyTestCase):
+class FloatConverter(TestCase):
     compiler = ''
-    def check_type_match_string(self,level=5):
+    @dec.slow
+    def test_type_match_string(self):
         s = c_spec.float_converter()
-        assert( not s.type_match('string') )
-    def check_type_match_int(self,level=5):
+        assert( not s.type_match('string'))
+    @dec.slow
+    def test_type_match_int(self):
         s = c_spec.float_converter()
         assert(not s.type_match(5))
-    def check_type_match_float(self,level=5):
+    @dec.slow
+    def test_type_match_float(self):
         s = c_spec.float_converter()
         assert(s.type_match(5.))
-    def check_type_match_complex(self,level=5):
+    @dec.slow
+    def test_type_match_complex(self):
         s = c_spec.float_converter()
         assert(not s.type_match(5.+1j))
-    def check_float_var_in(self,level=5):
+    @dec.slow
+    def test_float_var_in(self):
         mod_name = sys._getframe().f_code.co_name + self.compiler
         mod_name = unique_mod(test_dir,mod_name)
         mod = ext_tools.ext_module(mod_name)
@@ -141,7 +151,8 @@ class test_float_converter(NumpyTestCase):
             pass
 
 
-    def check_float_return(self,level=5):
+    @dec.slow
+    def test_float_return(self):
         mod_name = sys._getframe().f_code.co_name + self.compiler
         mod_name = unique_mod(test_dir,mod_name)
         mod = ext_tools.ext_module(mod_name)
@@ -158,21 +169,26 @@ class test_float_converter(NumpyTestCase):
         c = test(b)
         assert( c == 3.)
 
-class test_complex_converter(NumpyTestCase):
+class ComplexConverter(TestCase):
     compiler = ''
-    def check_type_match_string(self,level=5):
+    @dec.slow
+    def test_type_match_string(self):
         s = c_spec.complex_converter()
         assert( not s.type_match('string') )
-    def check_type_match_int(self,level=5):
+    @dec.slow
+    def test_type_match_int(self):
         s = c_spec.complex_converter()
         assert(not s.type_match(5))
-    def check_type_match_float(self,level=5):
+    @dec.slow
+    def test_type_match_float(self):
         s = c_spec.complex_converter()
         assert(not s.type_match(5.))
-    def check_type_match_complex(self,level=5):
+    @dec.slow
+    def test_type_match_complex(self):
         s = c_spec.complex_converter()
         assert(s.type_match(5.+1j))
-    def check_complex_var_in(self,level=5):
+    @dec.slow
+    def test_complex_var_in(self):
         mod_name = sys._getframe().f_code.co_name + self.compiler
         mod_name = unique_mod(test_dir,mod_name)
         mod = ext_tools.ext_module(mod_name)
@@ -195,7 +211,8 @@ class test_complex_converter(NumpyTestCase):
         except TypeError:
             pass
 
-    def check_complex_return(self,level=5):
+    @dec.slow
+    def test_complex_return(self):
         mod_name = sys._getframe().f_code.co_name + self.compiler
         mod_name = unique_mod(test_dir,mod_name)
         mod = ext_tools.ext_module(mod_name)
@@ -216,9 +233,10 @@ class test_complex_converter(NumpyTestCase):
 # File conversion tests
 #----------------------------------------------------------------------------
 
-class test_file_converter(NumpyTestCase):
+class FileConverter(TestCase):
     compiler = ''
-    def check_py_to_file(self,level=5):
+    @dec.slow
+    def test_py_to_file(self):
         import tempfile
         file_name = tempfile.mktemp()
         file = open(file_name,'w')
@@ -229,15 +247,16 @@ class test_file_converter(NumpyTestCase):
         file.close()
         file = open(file_name,'r')
         assert(file.read() == "hello bob")
-    def check_file_to_py(self,level=5):
+    @dec.slow
+    def test_file_to_py(self):
         import tempfile
         file_name = tempfile.mktemp()
         # not sure I like Py::String as default -- might move to std::sting
         # or just plain char*
         code = """
-               char* _file_name = (char*) file_name.c_str();
-               FILE* file = fopen(_file_name,"w");
-               return_val = file_to_py(file,_file_name,"w");
+               const char* _file_name = file_name.c_str();
+               FILE* file = fopen(_file_name, "w");
+               return_val = file_to_py(file, _file_name, "w");
                """
         file = inline_tools.inline(code,['file_name'], compiler=self.compiler,
                                    force=1)
@@ -250,16 +269,17 @@ class test_file_converter(NumpyTestCase):
 # Instance conversion tests
 #----------------------------------------------------------------------------
 
-class test_instance_converter(NumpyTestCase):
+class InstanceConverter(TestCase):
     pass
 
 #----------------------------------------------------------------------------
 # Callable object conversion tests
 #----------------------------------------------------------------------------
 
-class test_callable_converter(NumpyTestCase):
+class CallableConverter(TestCase):
     compiler=''
-    def check_call_function(self,level=5):
+    @dec.slow
+    def test_call_function(self):
         import string
         func = string.find
         search_str = "hello world hello"
@@ -277,36 +297,45 @@ class test_callable_converter(NumpyTestCase):
         desired = func(search_str,sub_str)
         assert(desired == actual)
 
-class test_sequence_converter(NumpyTestCase):
+class SequenceConverter(TestCase):
     compiler = ''
-    def check_convert_to_dict(self,level=5):
+    @dec.slow
+    def test_convert_to_dict(self):
         d = {}
         inline_tools.inline("",['d'],compiler=self.compiler,force=1)
-    def check_convert_to_list(self,level=5):
+    @dec.slow
+    def test_convert_to_list(self):
         l = []
         inline_tools.inline("",['l'],compiler=self.compiler,force=1)
-    def check_convert_to_string(self,level=5):
+    @dec.slow
+    def test_convert_to_string(self):
         s = 'hello'
         inline_tools.inline("",['s'],compiler=self.compiler,force=1)
-    def check_convert_to_tuple(self,level=5):
+    @dec.slow
+    def test_convert_to_tuple(self):
         t = ()
         inline_tools.inline("",['t'],compiler=self.compiler,force=1)
 
-class test_string_converter(NumpyTestCase):
+class StringConverter(TestCase):
     compiler = ''
-    def check_type_match_string(self,level=5):
+    @dec.slow
+    def test_type_match_string(self):
         s = c_spec.string_converter()
         assert( s.type_match('string') )
-    def check_type_match_int(self,level=5):
+    @dec.slow
+    def test_type_match_int(self):
         s = c_spec.string_converter()
         assert(not s.type_match(5))
-    def check_type_match_float(self,level=5):
+    @dec.slow
+    def test_type_match_float(self):
         s = c_spec.string_converter()
         assert(not s.type_match(5.))
-    def check_type_match_complex(self,level=5):
+    @dec.slow
+    def test_type_match_complex(self):
         s = c_spec.string_converter()
         assert(not s.type_match(5.+1j))
-    def check_var_in(self,level=5):
+    @dec.slow
+    def test_var_in(self):
         mod_name = 'string_var_in'+self.compiler
         mod_name = unique_mod(test_dir,mod_name)
         mod = ext_tools.ext_module(mod_name)
@@ -330,7 +359,8 @@ class test_string_converter(NumpyTestCase):
         except TypeError:
             pass
 
-    def check_return(self,level=5):
+    @dec.slow
+    def test_return(self):
         mod_name = 'string_return'+self.compiler
         mod_name = unique_mod(test_dir,mod_name)
         mod = ext_tools.ext_module(mod_name)
@@ -347,17 +377,20 @@ class test_string_converter(NumpyTestCase):
         c = test(b)
         assert( c == 'hello')
 
-class test_list_converter(NumpyTestCase):
+class ListConverter(TestCase):
     compiler = ''
-    def check_type_match_bad(self,level=5):
+    @dec.slow
+    def test_type_match_bad(self):
         s = c_spec.list_converter()
         objs = [{},(),'',1,1.,1+1j]
         for i in objs:
             assert( not s.type_match(i) )
-    def check_type_match_good(self,level=5):
+    @dec.slow
+    def test_type_match_good(self):
         s = c_spec.list_converter()
         assert(s.type_match([]))
-    def check_var_in(self,level=5):
+    @dec.slow
+    def test_var_in(self):
         mod_name = 'list_var_in'+self.compiler
         mod_name = unique_mod(test_dir,mod_name)
         mod = ext_tools.ext_module(mod_name)
@@ -380,7 +413,8 @@ class test_list_converter(NumpyTestCase):
         except TypeError:
             pass
 
-    def check_return(self,level=5):
+    @dec.slow
+    def test_return(self):
         mod_name = 'list_return'+self.compiler
         mod_name = unique_mod(test_dir,mod_name)
         mod = ext_tools.ext_module(mod_name)
@@ -398,7 +432,8 @@ class test_list_converter(NumpyTestCase):
         c = test(b)
         assert( c == ['hello'])
 
-    def check_speed(self,level=5):
+    @dec.slow
+    def test_speed(self):
         mod_name = 'list_speed'+self.compiler
         mod_name = unique_mod(test_dir,mod_name)
         mod = ext_tools.ext_module(mod_name)
@@ -458,17 +493,20 @@ class test_list_converter(NumpyTestCase):
         print 'python:', t2 - t1
         assert( sum1 == sum2 and sum1 == sum3)
 
-class test_tuple_converter(NumpyTestCase):
+class TupleConverter(TestCase):
     compiler = ''
-    def check_type_match_bad(self,level=5):
+    @dec.slow
+    def test_type_match_bad(self):
         s = c_spec.tuple_converter()
         objs = [{},[],'',1,1.,1+1j]
         for i in objs:
             assert( not s.type_match(i) )
-    def check_type_match_good(self,level=5):
+    @dec.slow
+    def test_type_match_good(self):
         s = c_spec.tuple_converter()
         assert(s.type_match((1,)))
-    def check_var_in(self,level=5):
+    @dec.slow
+    def test_var_in(self):
         mod_name = 'tuple_var_in'+self.compiler
         mod_name = unique_mod(test_dir,mod_name)
         mod = ext_tools.ext_module(mod_name)
@@ -491,7 +529,8 @@ class test_tuple_converter(NumpyTestCase):
         except TypeError:
             pass
 
-    def check_return(self,level=5):
+    @dec.slow
+    def test_return(self):
         mod_name = 'tuple_return'+self.compiler
         mod_name = unique_mod(test_dir,mod_name)
         mod = ext_tools.ext_module(mod_name)
@@ -511,25 +550,28 @@ class test_tuple_converter(NumpyTestCase):
         assert( c == ('hello',None))
 
 
-class test_dict_converter(NumpyTestCase):
+class DictConverter(TestCase):
     """ Base Class for dictionary conversion tests.
     """
 
     # Default string specifying the compiler to use.  While this is set
-    # in all sub-classes, this base test class is found by the test 
-    # infrastructure and run. Therefore, we give it a default value 
-    # so that it can run on its own. 
+    # in all sub-classes, this base test class is found by the test
+    # infrastructure and run. Therefore, we give it a default value
+    # so that it can run on its own.
     compiler=''
 
-    def check_type_match_bad(self,level=5):
+    @dec.slow
+    def test_type_match_bad(self):
         s = c_spec.dict_converter()
         objs = [[],(),'',1,1.,1+1j]
         for i in objs:
             assert( not s.type_match(i) )
-    def check_type_match_good(self,level=5):
+    @dec.slow
+    def test_type_match_good(self):
         s = c_spec.dict_converter()
         assert(s.type_match({}))
-    def check_var_in(self,level=5):
+    @dec.slow
+    def test_var_in(self):
         mod_name = 'dict_var_in'+self.compiler
         mod_name = unique_mod(test_dir,mod_name)
         mod = ext_tools.ext_module(mod_name)
@@ -552,7 +594,8 @@ class test_dict_converter(NumpyTestCase):
         except TypeError:
             pass
 
-    def check_return(self,level=5):
+    @dec.slow
+    def test_return(self):
         mod_name = 'dict_return'+self.compiler
         mod_name = unique_mod(test_dir,mod_name)
         mod = ext_tools.ext_module(mod_name)
@@ -570,87 +613,101 @@ class test_dict_converter(NumpyTestCase):
         c = test(b)
         assert( c['hello'] == 5)
 
-class test_msvc_int_converter(test_int_converter):
-    compiler = 'msvc'
-class test_unix_int_converter(test_int_converter):
-    compiler = ''
-class test_gcc_int_converter(test_int_converter):
-    compiler = 'gcc'
+# for compiler in compilers:
+    # for name,klass in globals().iteritems():
+    #     if name[:4]=="Test" and name[-9:] == "Converter":
+    #         exec("class %s%s(%s):\n    compiler = '%s'"%(name,compiler,name,compiler))
+# for converter in
+for _n in dir():
+    if _n[-9:]=='Converter':
+        if msvc_exists():
+            exec "class Test%sMsvc(%s):\n    compiler = 'msvc'"%(_n,_n)
+        else:
+            exec "class Test%sUnix(%s):\n    compiler = ''"%(_n,_n)
+        if gcc_exists():
+            exec "class Test%sGcc(%s):\n    compiler = 'gcc'"%(_n,_n)
 
-class test_msvc_float_converter(test_float_converter):
-    compiler = 'msvc'
+# class TestMsvcIntConverter(TestIntConverter):
+#     compiler = 'msvc'
+# class TestUnixIntConverter(TestIntConverter):
+#     compiler = ''
+# class TestGccIntConverter(TestIntConverter):
+#     compiler = 'gcc'
+#
+# class TestMsvcFloatConverter(TestFloatConverter):
+#     compiler = 'msvc'
+#
+# class TestMsvcFloatConverter(TestFloatConverter):
+#     compiler = 'msvc'
+# class TestUnixFloatConverter(TestFloatConverter):
+#     compiler = ''
+# class TestGccFloatConverter(TestFloatConverter):
+#     compiler = 'gcc'
+#
+# class TestMsvcComplexConverter(TestComplexConverter):
+#     compiler = 'msvc'
+# class TestUnixComplexConverter(TestComplexConverter):
+#     compiler = ''
+# class TestGccComplexConverter(TestComplexConverter):
+#     compiler = 'gcc'
+#
+# class TestMsvcFileConverter(TestFileConverter):
+#     compiler = 'msvc'
+# class TestUnixFileConverter(TestFileConverter):
+#     compiler = ''
+# class TestGccFileConverter(TestFileConverter):
+#     compiler = 'gcc'
+#
+# class TestMsvcCallableConverter(TestCallableConverter):
+#     compiler = 'msvc'
+# class TestUnixCallableConverter(TestCallableConverter):
+#     compiler = ''
+# class TestGccCallableConverter(TestCallableConverter):
+#     compiler = 'gcc'
+#
+# class TestMsvcSequenceConverter(TestSequenceConverter):
+#     compiler = 'msvc'
+# class TestUnixSequenceConverter(TestSequenceConverter):
+#     compiler = ''
+# class TestGccSequenceConverter(TestSequenceConverter):
+#     compiler = 'gcc'
+#
+# class TestMsvcStringConverter(TestStringConverter):
+#     compiler = 'msvc'
+# class TestUnixStringConverter(TestStringConverter):
+#     compiler = ''
+# class TestGccStringConverter(TestStringConverter):
+#     compiler = 'gcc'
+#
+# class TestMsvcListConverter(TestListConverter):
+#     compiler = 'msvc'
+# class TestUnixListConverter(TestListConverter):
+#     compiler = ''
+# class TestGccListConverter(TestListConverter):
+#     compiler = 'gcc'
+#
+# class TestMsvcTupleConverter(TestTupleConverter):
+#     compiler = 'msvc'
+# class TestUnixTupleConverter(TestTupleConverter):
+#     compiler = ''
+# class TestGccTupleConverter(TestTupleConverter):
+#     compiler = 'gcc'
+#
+# class TestMsvcDictConverter(TestDictConverter):
+#     compiler = 'msvc'
+# class TestUnixDictConverter(TestDictConverter):
+#     compiler = ''
+# class TestGccDictConverter(TestDictConverter):
+#     compiler = 'gcc'
+#
+# class TestMsvcInstanceConverter(TestInstanceConverter):
+#     compiler = 'msvc'
+# class TestUnixInstanceConverter(TestInstanceConverter):
+#     compiler = ''
+# class TestGccInstanceConverter(TestInstanceConverter):
+#     compiler = 'gcc'
 
-class test_msvc_float_converter(test_float_converter):
-    compiler = 'msvc'
-class test_unix_float_converter(test_float_converter):
-    compiler = ''
-class test_gcc_float_converter(test_float_converter):
-    compiler = 'gcc'
-
-class test_msvc_complex_converter(test_complex_converter):
-    compiler = 'msvc'
-class test_unix_complex_converter(test_complex_converter):
-    compiler = ''
-class test_gcc_complex_converter(test_complex_converter):
-    compiler = 'gcc'
-
-class test_msvc_file_converter(test_file_converter):
-    compiler = 'msvc'
-class test_unix_file_converter(test_file_converter):
-    compiler = ''
-class test_gcc_file_converter(test_file_converter):
-    compiler = 'gcc'
-
-class test_msvc_callable_converter(test_callable_converter):
-    compiler = 'msvc'
-class test_unix_callable_converter(test_callable_converter):
-    compiler = ''
-class test_gcc_callable_converter(test_callable_converter):
-    compiler = 'gcc'
-
-class test_msvc_sequence_converter(test_sequence_converter):
-    compiler = 'msvc'
-class test_unix_sequence_converter(test_sequence_converter):
-    compiler = ''
-class test_gcc_sequence_converter(test_sequence_converter):
-    compiler = 'gcc'
-
-class test_msvc_string_converter(test_string_converter):
-    compiler = 'msvc'
-class test_unix_string_converter(test_string_converter):
-    compiler = ''
-class test_gcc_string_converter(test_string_converter):
-    compiler = 'gcc'
-
-class test_msvc_list_converter(test_list_converter):
-    compiler = 'msvc'
-class test_unix_list_converter(test_list_converter):
-    compiler = ''
-class test_gcc_list_converter(test_list_converter):
-    compiler = 'gcc'
-
-class test_msvc_tuple_converter(test_tuple_converter):
-    compiler = 'msvc'
-class test_unix_tuple_converter(test_tuple_converter):
-    compiler = ''
-class test_gcc_tuple_converter(test_tuple_converter):
-    compiler = 'gcc'
-
-class test_msvc_dict_converter(test_dict_converter):
-    compiler = 'msvc'
-class test_unix_dict_converter(test_dict_converter):
-    compiler = ''
-class test_gcc_dict_converter(test_dict_converter):
-    compiler = 'gcc'
-
-class test_msvc_instance_converter(test_instance_converter):
-    compiler = 'msvc'
-class test_unix_instance_converter(test_instance_converter):
-    compiler = ''
-class test_gcc_instance_converter(test_instance_converter):
-    compiler = 'gcc'
-
-def setup_test_location():
+def setup_location():
     import tempfile
     #test_dir = os.path.join(tempfile.gettempdir(),'test_files')
     test_dir = tempfile.mktemp()
@@ -659,9 +716,9 @@ def setup_test_location():
     sys.path.insert(0,test_dir)
     return test_dir
 
-test_dir = setup_test_location()
+test_dir = setup_location()
 
-def teardown_test_location():
+def teardown_location():
     import tempfile
     test_dir = os.path.join(tempfile.gettempdir(),'test_files')
     if sys.path[0] == test_dir:
@@ -671,16 +728,17 @@ def teardown_test_location():
 def remove_file(name):
     test_dir = os.path.abspath(name)
 
-if not msvc_exists():
-    for _n in dir():
-        if _n[:10]=='test_msvc_': exec 'del '+_n
-else:
-    for _n in dir():
-        if _n[:10]=='test_unix_': exec 'del '+_n
-
-if not (gcc_exists() and msvc_exists() and sys.platform == 'win32'):
-    for _n in dir():
-        if _n[:9]=='test_gcc_': exec 'del '+_n
-
+# if not msvc_exists():
+#     for _n in dir():
+#         if _n[:8]=='TestMsvc': exec 'del '+_n
+# else:
+#     for _n in dir():
+#         if _n[:8]=='TestUnix': exec 'del '+_n
+#
+# if not (gcc_exists() and msvc_exists() and sys.platform == 'win32'):
+#     for _n in dir():
+#         if _n[:7]=='TestGcc': exec 'del '+_n
+#
 if __name__ == "__main__":
-    NumpyTest('weave.c_spec').run()
+    import nose
+    nose.run(argv=['', __file__])
